@@ -5,7 +5,7 @@ import java.io.*;
 
 class CDataReader
 {
-  
+ 
   public CDataReader()
   {
   }
@@ -16,12 +16,15 @@ class CDataReader
     try
     {
 //      URL source = new URL(getCodeBase(), fileName);
-      in = new DataInputStream(source.openStream());
+//TODO remove?      in = new DataInputStream(source.openStream());
+      in = new BufferedReader(new InputStreamReader(source.openStream()));
+ 
+
       mark = in.readLine();
 //      while(null != (aLine = in.readLine()))
 //        System.out.println(aLine);
     }
-    catch(Exception e) 
+    catch(Exception e)
     {
       in = null;
 //       e.printStackTrace();
@@ -44,7 +47,7 @@ class CDataReader
     }
     return null;
   }
-  
+ 
   public void Close()
   {
     try
@@ -59,5 +62,5 @@ class CDataReader
     }
     in = null;
   }
-  DataInputStream in;
+  BufferedReader in;
 };
