@@ -29,7 +29,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
 
     out = new AppletOutput(this);
     ResetInput();
- 
+
     String hintsfilename = getDocumentBase().toString();
     int slash = hintsfilename.lastIndexOf('/');
     if (slash >= 0)
@@ -259,9 +259,8 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
       }
       if (docbase.startsWith("http"))
       {
-        //jar:http://www.xs4all.nl/~apinkus/yacas.jar!/
         int pos = docbase.lastIndexOf("/");
-        String scriptBase = "jar:"+ docbase.substring(0,pos+1)+"yacas.jar!/";
+        String scriptBase = "jar:"+ docbase.substring(0,pos+1)+"yacas.jar!/scripts/";
         if (getParameter("debug") != null)
         {
           AddLineStatic(100, ""," '" + scriptBase + "'.", font, Color.red);
@@ -762,10 +761,10 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
         String key = inputLine.trim().substring(1);
 
         String prefix = "http://yacas.sourceforge.net/";
-          
+
         try {
             URI uri = new URI(prefix + "ref.html?" + key);
-            
+
             if (key.equals("license") || key.equals("licence"))
                 uri = new URI(prefix + "refprogchapter9.html");
             else if (key.equals("warranty"))
@@ -782,7 +781,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
             // it's a cold night in Hell
             return false;
         }
-        
+
         return true;
     }
 
@@ -935,7 +934,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
     {
       return size.height;
     }
-    Dimension size = new Dimension(320,240);   
+    Dimension size = new Dimension(320,240);
     int iIndent;
     private String iPrompt;
     private Font   iPromptFont;
@@ -1070,7 +1069,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
   {
     AddLineStatic(indent, "",text,  font, Color.black);
   }
- 
+
   Color iPromptColor = new Color(128,128,128);
   Font iPromptFont = new Font("Verdana", Font.PLAIN, 12);
   void AddLineStatic(int indent, String prompt, String text,  Font aFont, Color aColor)
@@ -1102,7 +1101,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
   public void paint(Graphics g)
   {
     CreateOffscreenImage();
- 
+
     // Render image
     paintToBitmap(offGra);
     // put the OffScreen image OnScreen
@@ -1122,7 +1121,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
       hintWindow.draw(5,(int)(d.getHeight()-context.FontHeight()-nr_total_lines*context.FontHeight()),context);
     }
   }
- 
+
   boolean thumbMoused = false;
   int scrollWidth = 16;
   int thumbPos = 0;
@@ -1264,7 +1263,7 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
 
   int cursorPos = 0;
   final int inset = 5;
- 
+
   final static String inputPrompt = "In> ";
   final static String outputPrompt = "Out> ";
 
@@ -1386,15 +1385,15 @@ public class ConsoleApplet extends Applet implements KeyListener, FocusListener,
     HintWindow hw = new HintWindow(fontsize);
     return hw;
   }
- 
+
   void AddHintLine(HintWindow hints, String aText, String aDescription)
   {
       hints.AddLine(aText);
       if (aDescription.length() > 0)
         hints.AddDescription(aDescription);
   }
- 
- 
+
+
   HintWindow TryToHint(String text, int length)
   {
     HintWindow hints = null;
