@@ -1,5 +1,7 @@
 package net.sf.yacas;
 
+import java.util.Set;
+import java.util.HashSet;
 
 class LispEnvironment
 {
@@ -12,9 +14,14 @@ class LispEnvironment
     iCurrentOutput = aCurrentOutput;
     iCurrentPrinter = new InfixPrinter(iPrefixOperators, iInfixOperators, iPostfixOperators, iBodiedOperators);
 
+    protected_symbols = new HashSet();
+
     iTrue = LispAtom.New(this,"True");
     iFalse = LispAtom.New(this,"False");
 
+    Protect(iTrue.String());
+    Protect(iTrue.String());
+
     iEndOfFile    = LispAtom.New(this,"EndOfFile");
     iEndStatement = LispAtom.New(this,";");
     iProgOpen     = LispAtom.New(this,"[");
@@ -71,6 +78,8 @@ class LispEnvironment
   LispOperators iPostfixOperators = new LispOperators();
   LispOperators iBodiedOperators = new LispOperators();
 
+  Set protected_symbols;
+
   int iEvalDepth = 0;
   int iMaxEvalDepth = 10000;
 
@@ -80,7 +89,7 @@ class LispEnvironment
   */
   class YacasArgStack
   {
- 
+
     //TODO appropriate constructor?
     public YacasArgStack(int aStackSize)
     {
@@ -264,7 +273,7 @@ class LispEnvironment
   }
   class LocalVariableFrame
   {
- 
+
       public LocalVariableFrame(LocalVariableFrame aNext, LispLocalVariable aFirst)
       {
         iNext = aNext;
@@ -286,7 +295,7 @@ class LispEnvironment
           t = next;
         }
       }
- 
+
 
       LocalVariableFrame iNext;
       LispLocalVariable iFirst;
@@ -315,9 +324,9 @@ class LispEnvironment
   LispTokenizer iXmlTokenizer = new XmlTokenizer();
 
   LispUserFunctions iUserFunctions = new LispUserFunctions();
- 
+
   String iError = null;
- 
+
   public void HoldArgument(String  aOperator, String aVariable) throws Exception
   {
     LispMultiUserFunction multiUserFunc = (LispMultiUserFunction)iUserFunctions.LookUp(aOperator);
@@ -333,6 +342,8 @@ class LispEnvironment
     }
   }
 
+
+
   public LispUserFunction UserFunction(LispPtr aArguments) throws Exception
   {
     LispMultiUserFunction multiUserFunc =
@@ -364,7 +375,7 @@ class LispEnvironment
     LispError.Check(userFunc != null, LispError.KLispErrInvalidArg);
     userFunc.UnFence();
   }
- 
+
   public LispMultiUserFunction MultiUserFunction(String aOperator) throws Exception
   {
     // Find existing multiuser func.
@@ -381,11 +392,11 @@ class LispEnvironment
     return multiUserFunc;
   }
 
- 
+
   public void DeclareRuleBase(String aOperator, LispPtr aParameters, boolean aListed) throws Exception
   {
     LispMultiUserFunction multiUserFunc = MultiUserFunction(aOperator);
- 
+
     // add an operator with this arity to the multiuserfunc.
     BranchingUserFunction newFunc;
     if (aListed)
@@ -411,9 +422,9 @@ class LispEnvironment
     // Get the specific user function with the right arity
     LispUserFunction userFunc = (LispUserFunction)multiUserFunc.UserFunc(aArity);
     LispError.Check(userFunc != null, LispError.KLispErrCreatingRule);
- 
+
     // Declare a new evaluation rule
- 
+
 
     if (LispStandard.IsTrue(this, aPredicate))
     {
@@ -448,12 +459,27 @@ class LispEnvironment
     // Get the specific user function with the right arity
     LispUserFunction userFunc = multiUserFunc.UserFunc(aArity);
     LispError.Check(userFunc != null, LispError.KLispErrCreatingRule);
- 
+
     // Declare a new evaluation rule
     userFunc.DeclarePattern(aPrecedence, aPredicate,aBody);
   }
 
 
+  void Protect(String symbol)
+  {
+      protected_symbols.add(symbol);
+  }
+
+  void UnProtect(String symbol)
+  {
+      protected_symbols.remove(symbol);
+  }
+
+  boolean IsProtected(String symbol)
+  {
+      return protected_symbols.contains(symbol);
+  }
+
   LispDefFiles iDefFiles = new LispDefFiles();
   InputDirectories iInputDirectories = new InputDirectories();
 
@@ -461,4 +487,3 @@ class LispEnvironment
   String iPrettyPrinter = null;
 
 }
-
