@@ -66,6 +66,8 @@ class LispError
           return "User interrupted calculation";
       if (aError ==  KLispErrNonBooleanPredicateInPattern)
           return "Predicate doesn't evaluate to a boolean in pattern";
+      if (aError ==  KLispErrSymbolProtected)
+          return "Trying to modify protected symbol";
      if (aError ==  KLispErrGenericFormat) return "Generic format";
     }
     return "Unspecified Error";
@@ -75,12 +77,12 @@ class LispError
     if (!hastobetrue)
     {
       String error = ErrorString(aError);//"Error number "+aError+" (//TODO FIXME still need to port over the string table)";
-      throw new Yacasexception(error);
+      throw new YacasException(error);
     }
   }
   static void RaiseError(String str) throws Exception
   {
-      throw new Yacasexception(str);
+      throw new YacasException(str);
   }
 
   public static void CheckNrArgs(int n, LispPtr aArguments, LispEnvironment aEnvironment) throws Exception
@@ -95,13 +97,13 @@ class LispError
   {
     if (aArguments.Get() == null)
     {
-      throw new Yacasexception("Error in compiled code.");
+      throw new YacasException("Error in compiled code.");
     }
     else
     {
 //TODO FIXME      ShowStack(aEnvironment);
       String error = ShowFunctionError(aArguments, aEnvironment) + "expected "+needed+" arguments, got "+passed;
-      throw new Yacasexception(error);
+      throw new YacasException(error);
 
 /*TODO FIXME
       LispChar str[20];
@@ -140,21 +142,21 @@ class LispError
         LispPtr arguments = YacasEvalCaller.ARGUMENT(aEnvironment,aStackTop,0);
         if (arguments.Get() == null)
         {
-          throw new Yacasexception("Error in compiled code\n");
+          throw new YacasException("Error in compiled code\n");
         }
         else
         {
           String error = "";
 //TODO FIXME          ShowStack(aEnvironment);
           error = error + ShowFunctionError(arguments, aEnvironment) + "generic error";
-          throw new Yacasexception(error);
+          throw new YacasException(error);
         }
       }
   }
   public static void LISPASSERT(boolean aPredicate) throws Exception
   {
     if (!aPredicate)
-      throw new Yacasexception("Assertion failed");
+      throw new YacasException("Assertion failed");
   }
   public static void CHK_ARG_CORE(LispEnvironment aEnvironment,int aStackTop,boolean aPredicate,int aArgNr) throws Exception
   {
@@ -175,7 +177,7 @@ class LispError
         LispPtr arguments = YacasEvalCaller.ARGUMENT(aEnvironment,aStackTop,0);
         if (arguments.Get() == null)
         {
-          throw new Yacasexception("Error in compiled code\n");
+          throw new YacasException("Error in compiled code\n");
         }
         else
         {
@@ -184,7 +186,7 @@ class LispError
           error = error + ShowFunctionError(arguments, aEnvironment) + "\nbad argument number "+aArgNr+"(counting from 1) : \n"+aErrorDescription + "\n";
           LispPtr arg = YacasEvalCaller.Argument(arguments,aArgNr);
           String strout;
- 
+
           error = error + "The offending argument ";
           strout = LispStandard.PrintExpression(arg, aEnvironment, 60);
           error = error + strout;
@@ -196,7 +198,7 @@ class LispError
           error = error + strout;
           error = error + "\n";
 
-          throw new Yacasexception(error);
+          throw new YacasException(error);
         }
     }
   }
@@ -231,8 +233,9 @@ class LispError
   static int KLispErrLibraryNotFound        = 26;
   static int KLispErrUserInterrupt          = 27;
   static int KLispErrNonBooleanPredicateInPattern = 28;
-  static int KLispErrGenericFormat          = 29;
-  static int KLispNrErrors                  = 30;
+  static int KLispErrSymbolProtected        = 29;
+  static int KLispErrGenericFormat          = 30;
+  static int KLispNrErrors                  = 31;
 
 
-}
\ No newline at end of file
+}
