@@ -4,10 +4,10 @@ package net.sf.yacas;
 /** \class LispPtrArray is similar to LispPtr, but implements an array
  *  of pointers to objects.
  */
-class LispPtrArray 
+class LispPtrArray
 {
   public LispPtrArray(int aSize,LispObject aInitialItem)
-	{
+  {
     iArray = new LispPtr[aSize];
     iSize = aSize;
     int i;
@@ -17,22 +17,18 @@ class LispPtrArray
       iArray[i].Set(aInitialItem);
     }
   }
-  void GrowTo(int aNewSize)
-	{
-	  //TODO FIXME
-	}
-    public int Size()
-	{
-	  return iSize;
-	}
-	public LispPtr GetElement(int aItem)
-	{
-	  return iArray[aItem];
-	}
-	public void SetElement(int aItem,LispObject aObject)
-	{
-	  iArray[aItem].Set(aObject);
-	}
-    int iSize;
-    LispPtr iArray[];
+  public int Size()
+  {
+    return iSize;
+  }
+  public LispPtr GetElement(int aItem)
+  {
+    return iArray[aItem];
+  }
+  public void SetElement(int aItem,LispObject aObject)
+  {
+    iArray[aItem].Set(aObject);
+  }
+  int iSize;
+  LispPtr iArray[];
 }
