@@ -3,32 +3,31 @@
 #include <string.h>
 #include <stdlib.h>
 
-char lastKey[128];
-char lastVal[16384];
-char lastDes[16384];
-char escaped[16384];
-char* escape(char* text)
-{
-  char* trg = escaped;
-  while (*text)
-  {
-    if (*text == '\'')
-    {
-      *trg++ = '\\';
-    }
-    *trg++ = *text++;
-  }
-  *trg = '\0';
+#include <string>
 
-  return escaped;
+std::string lastKey;
+std::string lastVal;
+std::string lastDes;
+
+std::string escape(const std::string& text)
+{
+    std::string r;
+    for (std::string::const_iterator i = text.begin(); i != text.end(); ++i) {
+        if (*i == '\'')
+            r += '\\';
+        r += *i;
+    }
+
+    return r;
 }
+
 FILE*fout;
 int first = 1;
 
 struct Example
 {
-  char name[32];
-  char example[512];
+    std::string name;
+    std::string example;
 };
 
 Example examples[1024];
@@ -38,7 +37,7 @@ int exampleCompare(const void *v1, const void *v2)
 {
   Example* e1 = (Example*)v1;
   Example* e2 = (Example*)v2;
-  return strcmp(e1->name,e2->name);
+  return strcmp(e1->name.c_str(), e2->name.c_str());
 }
 
 char *items[10];
@@ -65,29 +64,29 @@ void getfields(char* buffer)
   }
 }
 
-const char* findexample(char* name)
+const char* findexample(const char* name)
 {
   Example key;
-  strcpy(key.name,name);
+  key.name = name;
   void *found = bsearch(&key, examples, nrExamples, sizeof(Example),exampleCompare);
   if (found)
-    return ((Example*) found)->example;
+      return ((Example*) found)->example.c_str();
   return "";
 }
 
 void WriteLine()
 {
-  if (lastKey[0])
+  if (!lastKey.empty())
   {
     if (!first)
     {
       fprintf(fout,",\n");
     }
     first = 0;
-    fprintf(fout,"\'%s\',\n",escape(lastKey));
-    fprintf(fout,"\'<ul>%s</ul>\',\n",escape(lastVal));
-    fprintf(fout,"\'%s\',\n",escape(lastDes));
-    fprintf(fout,"\'%s\'\n",findexample(lastKey)); // placeholder for examples
+    fprintf(fout,"\'%s\',\n",escape(lastKey).c_str());
+    fprintf(fout,"\'<ul>%s</ul>\',\n",escape(lastVal).c_str());
+    fprintf(fout,"\'%s\',\n",escape(lastDes).c_str());
+    fprintf(fout,"\'%s\'\n",findexample(lastKey.c_str())); // placeholder for examples
     lastKey[0] = '\0';
     lastVal[0] = '\0';
     lastDes[0] = '\0';
@@ -118,8 +117,8 @@ int main(int argc, char** argv)
       getfields(buffer);
       if (nrItems>1)
       {
-        strcpy(examples[nrExamples].name,items[0]);
-        strcpy(examples[nrExamples].example,items[1]);
+        examples[nrExamples].name = items[0];
+        examples[nrExamples].example = items[1];
         nrExamples++;
       }
     } 
@@ -129,10 +128,10 @@ int main(int argc, char** argv)
       int i;
       for (i=0;i<nrExamples-1;i++)
       {
-        if (!strcmp(examples[i].name,examples[i+1].name))
-        {
-          printf("ERROR generating autocompleter array: duplicate entries found for %s\n",examples[i].name);
-          exit(-1);
+          if (!strcmp(examples[i].name.c_str(), examples[i+1].name.c_str()))
+          {
+              printf("ERROR generating autocompleter array: duplicate entries found for %s\n",examples[i].name.c_str());
+              exit(-1);
         }
       }
     }
@@ -150,39 +149,39 @@ int main(int argc, char** argv)
     getfields(buffer);
     if (nrItems>3)
     {
-      if (strcmp(lastKey,items[1]))
+      if (strcmp(lastKey.c_str(), items[1]))
       {
         WriteLine();
-        strcpy(lastKey,items[1]);
-        strcpy(lastVal,"");
-        if (!strncmp(lastKey,items[2],strlen(lastKey)))
+        lastKey = items[1];
+        lastVal = "";
+        if (!strncmp(lastKey.c_str(),items[2],lastKey.length()))
         {
-          strcat(lastVal,"<li><b>");
-          strcat(lastVal,lastKey);
-          strcat(lastVal,"</b>");
-          strcat(lastVal,&items[2][strlen(lastKey)]);
+          lastVal += "<li><b>";
+          lastVal += lastKey;
+          lastVal += "</b>";
+          lastVal += &items[2][lastKey.length()];
         }
         else
         {
-          strcat(lastVal,items[2]);
+          lastVal += items[2];
         }
-        strcpy(lastDes,items[3]);
+        lastDes = items[3];
       }
       else
       {
 //        strcat(lastVal,", \\n");
-        if (!strncmp(lastKey,items[2],strlen(lastKey)))
+        if (!strncmp(lastKey.c_str(),items[2],lastKey.length()))
         {
-          strcat(lastVal,"<li><b>");
-          strcat(lastVal,lastKey);
-          strcat(lastVal,"</b>");
-          strcat(lastVal,&items[2][strlen(lastKey)]);
+          lastVal += "<li><b>";
+          lastVal += lastKey;
+          lastVal += "</b>";
+          lastVal += &items[2][lastKey.length()];
         }
         else
         {
-          strcat(lastVal,items[2]);
+          lastVal += items[2];
         }
-        strcat(lastDes,items[3]);
+        lastDes += items[3];
       }
     }
   } 
