@@ -1,4 +1,3 @@
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -20,7 +19,7 @@ char* indexOf(char* haystack, const char* needle)
     }
     pos++;
   }
-  return NULL;
+  return nullptr;
 }
 
 int main(int argc, char** argv)
@@ -30,7 +29,7 @@ int main(int argc, char** argv)
   char* inName  = argv[1];
   char* outName = argv[2];
 
-  char* inbuffer = NULL;
+  char* inbuffer = nullptr;
 
   FILE* fin = fopen(inName,"rb");
   if (!fin)
@@ -56,19 +55,19 @@ int main(int argc, char** argv)
   }
   inbuffer[size] = '\0';
   fclose(fin);
-  
+
   FILE* fout=fopen(outName,"wb");
   if (!fout)
   {
     fprintf(stderr,"Could not open file %s for writing\n",outName);
     exit(-1);
   }
-  
-  
+
+
   char* pos;
-  
+
   pos = indexOf(inbuffer,"{{code:");
-  while (pos != NULL)
+  while (pos != nullptr)
   {
     char* start = pos+7;
     char* end = indexOf(start,":code}}");
@@ -81,7 +80,7 @@ int main(int argc, char** argv)
   }
 
   pos = indexOf(inbuffer,"{{test:");
-  while (pos != NULL)
+  while (pos != nullptr)
   {
     char* start = pos+7;
     char* end = indexOf(start,":test}}");
@@ -97,6 +96,6 @@ int main(int argc, char** argv)
 
   free(inbuffer);
   fclose(fout);
-  
+
   return 0;
 }
