@@ -448,7 +448,7 @@ tabberObj.prototype.navSetActive = function(tabberIndex)
       elem.innerHTML = 
 "<center>"+
 "<h1>Loading page, one moment please.</h1>"+
-"<p style='width:100% ; height:50% ; background-image:url(\"back.gif\"); ' >"+
+"<p style='width:100% ; height:50% ; ' >"+
 "If this page includes a Yacas calculation center, and you are visiting this page for the first time, "+
 "it could take a minute to download the program."+
 "</p></center>";
@@ -490,7 +490,7 @@ function initTabPageIFrame(id)
       }
     }
 
-    elem.innerHTML = '<iframe name="'+fields[0]+'" id="'+fields[0]+'" frameborder="0" width="'+width+'" height="80%" src="'+target+'"></iframe>';
+    elem.innerHTML = '<iframe name="'+fields[0]+'" id="'+fields[0]+'" frameborder="0" width="'+width+'" height="85%" src="'+target+'"></iframe>';
   }
 }
 
