@@ -1,36 +1,31 @@
 
-#include "yacasprivate.h"
-#include "lispassert.h"
-#include "infixparser.h"
-#include "lispatom.h"
-#include "standard.h"
-#include "lisperror.h"
-#include "errors.h"
-
-
-//#define RAISE_PARSE_ERROR { iError = LispTrue; RaiseError("Error parsing expression, near token %s",iLookAhead->String()); }
+#include "yacas/yacasprivate.h"
+#include "yacas/infixparser.h"
+#include "yacas/lispatom.h"
+#include "yacas/standard.h"
+#include "yacas/lisperror.h"
+#include "yacas/errors.h"
 
+#include <cassert>
+#include <string>
 
 void ParsedObject::Fail()
 {
-   iError = LispTrue;
    if (iLookAhead && iLookAhead->c_str())
-   {
-//    if (IsSymbolic(iLookAhead->c_str()[0]))
-//       RaiseError("Error parsing expression, near token %s (maybe you forgot a space between two operators?)",iLookAhead->c_str());
-      RaiseError("Error parsing expression, near token %s",iLookAhead->c_str());
-   }
-   RaiseError("Error parsing expression");
-}
-#define RAISE_PARSE_ERROR Fail()
+       throw LispErrGeneric(std::string("Error parsing expression, near token ") +  std::string(iLookAhead->c_str()));
 
-InfixParser::InfixParser(LispTokenizer& aTokenizer, LispInput& aInput,
+   throw LispErrGeneric("Error parsing expression");
+}
+
+
+InfixParser::InfixParser(LispTokenizer& aTokenizer,
+                         LispInput& aInput,
                          LispEnvironment& aEnvironment,
                          LispOperators& aPrefixOperators,
                          LispOperators& aInfixOperators,
                          LispOperators& aPostfixOperators,
-                         LispOperators& aBodiedOperators)
-    : LispParser( aTokenizer,  aInput,aEnvironment),
+                         LispOperators& aBodiedOperators):
+    LispParser(aTokenizer, aInput, aEnvironment),
     iPrefixOperators(aPrefixOperators),
     iInfixOperators(aInfixOperators),
     iPostfixOperators(aPostfixOperators),
@@ -38,18 +33,11 @@ InfixParser::InfixParser(LispTokenizer& aTokenizer, LispInput& aInput,
 {
 }
 
-InfixParser::~InfixParser()
-{
-}
-
-
-
-
 void InfixParser::Parse(LispPtr& aResult )
 {
-//    iEnvironment = &aEnvironment;
     ParseCont(aResult);
 }
+
 void InfixParser::ParseCont(LispPtr& aResult)
 {
     ParsedObject object(*this);
@@ -58,34 +46,43 @@ void InfixParser::ParseCont(LispPtr& aResult)
     return ;
 }
 
-void LispOperators::SetOperator(LispInt aPrecedence, LispString * aString)
+void LispOperators::SetOperator(LispInt aPrecedence, const LispString* aString)
 {
-    LispInFixOperator op(aPrecedence);
-    SetAssociation(op, aString);
+    _map[aString] = LispInFixOperator(aPrecedence);
 }
 
 
-void LispOperators::SetRightAssociative(LispString * aString)
+void LispOperators::SetRightAssociative(const LispString* aString)
 {
-    LispInFixOperator* op = LookUp(aString);
-    Check(op,KLispErrNotAnInFixOperator);
-    op->SetRightAssociative();
+    auto i = _map.find(aString);
+    if (i == _map.end())
+        throw LispErrNotAnInFixOperator();
+    i->second.SetRightAssociative();
 }
 
-void LispOperators::SetLeftPrecedence(LispString * aString,LispInt aPrecedence)
+void LispOperators::SetLeftPrecedence(const LispString* aString, LispInt aPrecedence)
 {
-    LispInFixOperator* op = LookUp(aString);
-    Check(op,KLispErrNotAnInFixOperator);
-    op->SetLeftPrecedence(aPrecedence);
+    auto i = _map.find(aString);
+    if (i == _map.end())
+        throw LispErrNotAnInFixOperator();
+    i->second.SetLeftPrecedence(aPrecedence);
 }
 
-void LispOperators::SetRightPrecedence(LispString * aString,LispInt aPrecedence)
+void LispOperators::SetRightPrecedence(const LispString* aString, LispInt aPrecedence)
 {
-    LispInFixOperator* op = LookUp(aString);
-    Check(op,KLispErrNotAnInFixOperator);
-    op->SetRightPrecedence(aPrecedence);
+    auto i = _map.find(aString);
+    if (i == _map.end())
+        throw LispErrNotAnInFixOperator();
+    i->second.SetRightPrecedence(aPrecedence);
 }
 
+LispInFixOperator* LispOperators::LookUp(const LispString* aString)
+{
+    const auto i = _map.find(aString);
+    if (i != _map.end())
+        return &i->second;
+    return 0;
+}
 
 void ParsedObject::ReadToken()
 {
@@ -93,13 +90,14 @@ void ParsedObject::ReadToken()
     iLookAhead = iParser.iTokenizer.NextToken(iParser.iInput,
                                               iParser.iEnvironment.HashTable());
     if (iLookAhead->c_str()[0] == '\0')
-        iEndOfFile=LispTrue;
+        iEndOfFile=true;
 }
 
-void ParsedObject::MatchToken(LispString * aToken)
+void ParsedObject::MatchToken(const LispString* aToken)
 {
     if (aToken != iLookAhead)
-          RAISE_PARSE_ERROR; // iError=LispTrue;
+          Fail();
+
     ReadToken();
 }
 
@@ -115,63 +113,53 @@ void ParsedObject::Parse()
     ReadExpression(KMaxPrecedence);  // least precedence
 
     if (iLookAhead != iParser.iEnvironment.iEndStatement->String())
-    {
-      RAISE_PARSE_ERROR; // iError = LispTrue;
-    }
-    if (iError)
-    {
-        while ((*iLookAhead)[0] != '\0' && iLookAhead != iParser.iEnvironment.iEndStatement->String())
-        {
-            ReadToken();
-        }
-    }
-
-    if (iError)
-    {
-        iResult = (NULL);
-    }
-    Check(!iError,KLispErrInvalidExpression);
+        Fail();
 }
 
 
 void ParsedObject::Combine(LispInt aNrArgsToCombine)
 {
     LispPtr subList(LispSubList::New(iResult));
+
     DBG_( subList->SetFileAndLine(
-    iParser.iInput.Status().FileName(),
-    iParser.iInput.Status().LineNumber() ); )
+              iParser.iInput.Status().FileName(),
+              iParser.iInput.Status().LineNumber() ); );
 
   // TODO: woof -- such ugliness!
 
     LispIterator iter(iResult);
     for (LispInt i=0; i<aNrArgsToCombine; i++, ++iter)
-    {
-    if (!iter.getObj()) { RAISE_PARSE_ERROR; return; } // iError = LispTrue;
-  }
-  if (!iter.getObj()) { RAISE_PARSE_ERROR; return; } // iError = LispTrue;
-    subList->Nixed() = (*++iter);
-    *iter = (NULL);
+        if (!iter.getObj())
+            Fail();
 
-  InternalReverseList((*subList->SubList())->Nixed(),  // TODO: woof
-                     (*subList->SubList())->Nixed());
+    if (!iter.getObj())
+        Fail();
+
+    subList->Nixed() = (*++iter);
+    *iter = nullptr;
+
+    InternalReverseList((*subList->SubList())->Nixed(),  // TODO: woof
+                        (*subList->SubList())->Nixed());
     iResult = subList;
 }
 
 void ParsedObject::GetOtherSide(LispInt aNrArgsToCombine, LispInt depth)
 {
-    LispString * theOperator = iLookAhead;
+    const LispString* theOperator = iLookAhead;
     MatchToken(iLookAhead);
     ReadExpression(depth);
     InsertAtom(theOperator);
     Combine(aNrArgsToCombine);
 }
 
-void ParsedObject::InsertAtom(LispString * aString)
+void ParsedObject::InsertAtom(const LispString* aString)
 {
-    LispPtr ptr(LispAtom::New(iParser.iEnvironment,aString->c_str()));
+    LispPtr ptr(LispAtom::New(iParser.iEnvironment, *aString));
+
     DBG_( ptr->SetFileAndLine(
-    iParser.iInput.Status().FileName(),
-    iParser.iInput.Status().LineNumber() ); )
+              iParser.iInput.Status().FileName(),
+              iParser.iInput.Status().LineNumber() ); );
+
     ptr->Nixed() = (iResult);
     iResult = (ptr);
 }
@@ -192,13 +180,11 @@ void ParsedObject::ReadExpression(LispInt depth)
             ReadExpression(KMaxPrecedence);
             // Match closing bracket
             if (iLookAhead != iParser.iEnvironment.iProgClose->String())
-            {
-                RaiseError("Expecting a ] close bracket for program block, but got %s instead",iLookAhead->c_str()); // RAISE_PARSE_ERROR; // iError = LispTrue;
-                return;
-            }
+                throw LispErrGeneric(std::string("Expecting a ] close bracket for program block, but got ") + *iLookAhead + std::string(" instead"));
+
             MatchToken(iLookAhead);
             // Build into Ntn(...)
-            LispString * theOperator = iParser.iEnvironment.iNth->String();
+            const LispString* theOperator = iParser.iEnvironment.iNth->String();
             InsertAtom(theOperator);
             Combine(2);
         }
@@ -210,15 +196,15 @@ void ParsedObject::ReadExpression(LispInt depth)
 //printf("op [%s]\n",iLookAhead->c_str());
               if (IsSymbolic((*iLookAhead)[0]))
               {
-                LispInt origlen = iLookAhead->Size()-1;
+                LispInt origlen = iLookAhead->size();
                 LispInt len = origlen;
 //printf("IsSymbolic, len=%d\n",len);
 
                 while (len>1)
                 {
                   len--;
-                  LispString * lookUp =
-                    iParser.iEnvironment.HashTable().LookUpCounted(iLookAhead->c_str(),len);
+                  const LispString* lookUp =
+                    iParser.iEnvironment.HashTable().LookUp(iLookAhead->substr(0, len));
 
 //printf("trunc %s\n",lookUp->c_str());
                   op = iParser.iInfixOperators.LookUp(lookUp);
@@ -226,8 +212,8 @@ void ParsedObject::ReadExpression(LispInt depth)
                   if (op)
                   {
 
-                    LispString * lookUpRight =
-                      iParser.iEnvironment.HashTable().LookUpCounted(&(iLookAhead->c_str()[len]),origlen-len);
+                    const LispString* lookUpRight =
+                        iParser.iEnvironment.HashTable().LookUp(iLookAhead->substr(len, origlen - len));
 
 //printf("right: %s (%d)\n",lookUpRight->c_str(),origlen-len);
 
@@ -241,7 +227,7 @@ void ParsedObject::ReadExpression(LispInt depth)
 //printf("Pushhback %s\n",&input.StartPtr()[input.Position()]);
                       break;
                     }
-                    else op=NULL;
+                    else op=nullptr;
                   }
                 }
                 if (!op) return;
@@ -275,7 +261,7 @@ void ParsedObject::ReadAtom()
     op = iParser.iPrefixOperators.LookUp(iLookAhead);
     if (op)
     {
-        LispString * theOperator = iLookAhead;
+        const LispString* theOperator = iLookAhead;
         MatchToken(iLookAhead);
         {
             ReadExpression(op->iPrecedence);
@@ -306,12 +292,11 @@ void ParsedObject::ReadAtom()
             }
             else if (iLookAhead != iParser.iEnvironment.iListClose->String())
             {
-                RaiseError("Expecting a } close bracket for a list, but got %s instead",iLookAhead->c_str()); // RAISE_PARSE_ERROR; // iError = LispTrue;
-                return;
+                throw LispErrGeneric(std::string("Expecting a } close bracket for program block, but got ") + std::string(iLookAhead->c_str()) + std::string(" instead"));
             }
         }
         MatchToken(iLookAhead);
-        LispString * theOperator = iParser.iEnvironment.iList->String();
+        const LispString* theOperator = iParser.iEnvironment.iList->String();
         InsertAtom(theOperator);
         Combine(nrargs);
 
@@ -338,12 +323,11 @@ void ParsedObject::ReadAtom()
             }
             else
             {
-                RaiseError("Expecting ; end of statement in program block, but got %s instead",iLookAhead->c_str()); // RAISE_PARSE_ERROR; // iError = LispTrue;
-                return;
+                throw LispErrGeneric(std::string("Expecting ; end of statement in program block, but got ") + std::string(iLookAhead->c_str()) + std::string(" instead"));
             }
         }
         MatchToken(iLookAhead);
-        LispString * theOperator = iParser.iEnvironment.iProg->String();
+        const LispString* theOperator = iParser.iEnvironment.iProg->String();
         InsertAtom(theOperator);
 
         Combine(nrargs);
@@ -355,7 +339,7 @@ void ParsedObject::ReadAtom()
     // Else we have an atom.
     else
     {
-        LispString * theOperator = iLookAhead;
+        const LispString* theOperator = iLookAhead;
         MatchToken(iLookAhead);
 
         LispInt nrargs=-1;
@@ -374,8 +358,7 @@ void ParsedObject::ReadAtom()
                 }
                 else if (iLookAhead != iParser.iEnvironment.iBracketClose->String())
                 {
-                    RaiseError("Expecting ) closing bracket for sub-expression, but got %s instead",iLookAhead->c_str()); // RAISE_PARSE_ERROR; // iError = LispTrue;
-                    return;
+                    throw LispErrGeneric(std::string("Expecting a ) closing bracket for sub-expression, but got ") + std::string(iLookAhead->c_str()) + std::string(" instead"));
                 }
             }
             MatchToken(iLookAhead);
@@ -395,7 +378,7 @@ void ParsedObject::ReadAtom()
 
     // Parse postfix operators
 
-    while ((op = iParser.iPostfixOperators.LookUp(iLookAhead)))
+    while (iParser.iPostfixOperators.LookUp(iLookAhead))
     {
         InsertAtom(iLookAhead);
         MatchToken(iLookAhead);
@@ -404,18 +387,15 @@ void ParsedObject::ReadAtom()
 }
 
 
-void InfixPrinter::WriteToken(LispOutput& aOutput, const LispChar * aString)
+void InfixPrinter::WriteToken(std::ostream& aOutput, const std::string& aString)
 {
     if (IsAlNum(iPrevLastChar) && (IsAlNum(aString[0]) || aString[0]=='_'))
-    {
-        aOutput.Write(" ");
-    }
+        aOutput.put(' ');
     else if (IsSymbolic(iPrevLastChar) && IsSymbolic(aString[0]))
-    {
-        aOutput.Write(" ");
-    }
-    aOutput.Write(aString);
-  RememberLastChar(aString[PlatStrLen(aString)-1]);
+        aOutput.put(' ');
+
+    aOutput.write(aString.c_str(), aString.size());
+    RememberLastChar(aString.back());
 }
 
 void InfixPrinter::RememberLastChar(LispChar aChar)
@@ -423,31 +403,35 @@ void InfixPrinter::RememberLastChar(LispChar aChar)
   iPrevLastChar = aChar;
 }
 
-void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
-                       LispEnvironment& aEnvironment)
+void InfixPrinter::Print(
+    const LispPtr& aExpression,
+    std::ostream& aOutput,
+    LispEnvironment& aEnvironment)
 {
     iCurrentEnvironment = &aEnvironment;
     Print(aExpression, aOutput, KMaxPrecedence);
 }
 
-void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
-                         LispInt iPrecedence)
+void InfixPrinter::Print(
+    const LispPtr& aExpression,
+    std::ostream& aOutput,
+    LispInt iPrecedence)
 {
-    LISPASSERT(aExpression);
+    assert(aExpression);
 
-    LispString * string = aExpression->String();
+    const LispString* string = aExpression->String();
     if (string)
     {
         LispInt bracket=0;
         if (iPrecedence<KMaxPrecedence &&
             (*string)[0] == '-' &&
-            (IsDigit((*string)[1]) || (*string)[1] == '.')
+            (std::isdigit((*string)[1]) || (*string)[1] == '.')
            )
         {
             bracket=1;
         }
         if (bracket) WriteToken(aOutput,"(");
-        WriteToken(aOutput,string->c_str());
+        WriteToken(aOutput, *string);
         if (bracket) WriteToken(aOutput,")");
         return;
     }
@@ -460,29 +444,25 @@ void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
     }
 
     LispPtr* subList = aExpression->SubList();
-    Check(subList, KLispErrUnprintableToken);
-    if (!subList)
-    {
-        WriteToken(aOutput,"( )");
-    }
-    else
-    {
+    if (!subList) {
+        throw LispErrUnprintableToken();
+    } else {
         LispInt length = InternalListLength(*subList);
         string = (*subList)->String();
         LispInFixOperator* prefix  = iPrefixOperators.LookUp(string);
         LispInFixOperator* infix   = iInfixOperators.LookUp(string);
         LispInFixOperator* postfix = iPostfixOperators.LookUp(string);
         LispInFixOperator* bodied  = iBodiedOperators.LookUp(string);
-        LispInFixOperator* op = NULL;
+        LispInFixOperator* op = nullptr;
 
         if (length!=2)
         {
-            prefix=NULL;
-            postfix=NULL;
+            prefix=nullptr;
+            postfix=nullptr;
         }
         if (length!=3)
         {
-            infix=NULL;
+            infix=nullptr;
         }
         if (prefix  )  op=prefix;
         if (postfix )  op=postfix;
@@ -490,8 +470,8 @@ void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
 
         if (op)
         {
-            LispPtr* left  = NULL;
-            LispPtr* right = NULL;
+            LispPtr* left  = nullptr;
+            LispPtr* right = nullptr;
 
             if (prefix)
             {
@@ -516,7 +496,7 @@ void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
             }
             if (left)
                 Print(*left, aOutput,op->iLeftPrecedence);
-            WriteToken(aOutput,string->c_str());
+            WriteToken(aOutput, *string);
             if (right)
                 Print(*right, aOutput,op->iRightPrecedence);
             if (iPrecedence < op->iPrecedence)
@@ -556,17 +536,17 @@ void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
             }
             else
             {
-                LispInt bracket = LispFalse;
+                LispInt bracket = false;
                 if (bodied)
                 {
 //printf("%d > %d\n",iPrecedence, bodied->iPrecedence);
                   if (iPrecedence < bodied->iPrecedence)
-                    bracket = LispTrue;
+                    bracket = true;
                 }
                 if (bracket) WriteToken(aOutput,"(");
                 if (string)
                 {
-                  WriteToken(aOutput,string->c_str());
+                  WriteToken(aOutput, *string);
                 }
                 else
                 {
@@ -593,8 +573,10 @@ void InfixPrinter::Print(LispPtr& aExpression, LispOutput& aOutput,
                         WriteToken(aOutput,",");
                 }
                 WriteToken(aOutput,")");
-                if (iter.getObj())
+                if (iter.getObj()) {
+                    assert(bodied);
                     Print(*iter, aOutput, bodied->iPrecedence);
+                }
 
                 if (bracket) WriteToken(aOutput,")");
             }
