@@ -1,34 +1,26 @@
 
-#include "yacasprivate.h"
-#include "lispatom.h"
-#include "lispassert.h"
-#include "lisperror.h"
-#include "numbers.h"
-#include "standard.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/lispatom.h"
+#include "yacas/lisperror.h"
+#include "yacas/numbers.h"
+#include "yacas/standard.h"
 
+#include <algorithm>
+#include <cassert>
 
 /// construct an atom from a string representation.
-LispObject* LispAtom::New(LispEnvironment& aEnvironment, const LispChar * aString)
+LispObject* LispAtom::New(LispEnvironment& aEnvironment, const std::string& aString)
 {
-  LispObject* self;
-  if (IsNumber(aString,LispTrue))  // check if aString is a number (int or float)
-  {
-    /// construct a number from a decimal string representation (also create a number object)
-    self = NEW LispNumber(NEW LispString(aString), aEnvironment.Precision());
-  }
-  else
-  {
-    self = NEW LispAtom(aEnvironment.HashTable().LookUp(aString));
-  }
-  Check(self!=NULL,KLispErrNotEnoughMemory);
-  return self;
+    if (IsNumber(aString.c_str(),true))  // check if aString is a number (int or float)
+        return NEW LispNumber(NEW LispString(aString), aEnvironment.Precision());
+
+    return NEW LispAtom(aEnvironment.HashTable().LookUp(aString));
 }
 
-LispAtom::LispAtom(LispString * aString) : iString(aString)
+LispAtom::LispAtom(const LispString* aString) : iString(aString)
 {
-    LISPASSERT(aString!=NULL);
+    assert(aString);
     ++aString->iReferenceCount;
-    CHECKPTR(iString);
 }
 
 LispAtom::LispAtom(const LispAtom& other) : ASuper(other), iString(other.iString)
@@ -42,9 +34,9 @@ LispAtom::~LispAtom()
 }
 
 
-LispString * LispAtom::String()
+const LispString* LispAtom::String()
 {
-    CHECKPTR(iString);
+    assert(iString);
     return iString;
 }
 
@@ -54,7 +46,10 @@ LispString * LispAtom::String()
 LispSubList* LispSubList::New(LispObject* aSubList)
 {
     LispSubList* self = NEW LispSubList(aSubList);
-    Check(self!=NULL,KLispErrNotEnoughMemory);
+
+    if (!self)
+      throw LispErrNotEnoughMemory();
+
     return self;
 }
 
@@ -67,7 +62,7 @@ LispSubList::~LispSubList()
     {
         LispPtr next;
         LispIterator iter(iSubList);
-        LispBoolean busy = (iter.getObj()->iReferenceCount == 1);
+        bool busy = (iter.getObj()->iReferenceCount == 1);
         while (busy) // while there are things to delete...
         {
       // TODO: woof -- fix this ugliness!
@@ -77,16 +72,16 @@ LispSubList::~LispSubList()
             // Separate out the current element...
             if (iter.getObj()->iReferenceCount == 1)
             {// Destructive operation only if necessary...
-                iter.getObj()->Nixed() = (NULL);
+                iter.getObj()->Nixed() = (nullptr);
                 // And delete it.
-                (*iter) = (NULL);
+                (*iter) = (nullptr);
             }
             else
-                busy=LispFalse;
+                busy=false;
             next = (nextToDelete);
             iter = next;
             if (!iter.getObj())
-                busy=LispFalse;
+                busy=false;
         }
     }
 }
@@ -97,24 +92,23 @@ LispSubList::~LispSubList()
 LispGenericClass* LispGenericClass::New(GenericClass* aClass)
 {
     LispGenericClass* self = NEW LispGenericClass(aClass);
-    Check(self!=NULL,KLispErrNotEnoughMemory);
+
+    if (!self)
+      throw LispErrNotEnoughMemory();
+
     return self;
 }
 
 LispGenericClass::LispGenericClass(GenericClass* aClass) : iClass(aClass)
 {
-    LISPASSERT(aClass!=NULL);
+    assert(aClass!=nullptr);
     aClass->iReferenceCount++;
 }
 
 LispGenericClass::~LispGenericClass()
 {
-    iClass->iReferenceCount--;
-    if (iClass->iReferenceCount == 0)
-    {
+    if (--iClass->iReferenceCount == 0)
         delete iClass;
-    }
-    iClass=NULL;
 }
 
 GenericClass* LispGenericClass::Generic()
@@ -131,10 +125,10 @@ LispString * LispNumber::String()
 {
   if (!iString)
   {
-    LISPASSERT(iNumber.ptr());  // either the string is null or the number but not both
+    assert(iNumber.ptr());  // either the string is null or the number but not both
     LispString *str = NEW LispString;
     // export the current number to string and store it as LispNumber::iString
-    iNumber->ToString(*str, bits_to_digits(MAX(1,iNumber->GetPrecision()),BASE10), BASE10);
+    iNumber->ToString(*str, bits_to_digits(std::max(1,iNumber->GetPrecision()),BASE10), BASE10);
     iString = str;
   }
   return iString;
@@ -146,7 +140,7 @@ BigNumber* LispNumber::Number(LispInt aBasePrecision)
 {
   if (!iNumber)
   {  // create and store a BigNumber out of string
-    LISPASSERT(iString.ptr());
+    assert(iString.ptr());
     RefPtr<LispString> str;
     str = iString;
     // aBasePrecision is in digits, not in bits, ok
