@@ -1,13 +1,15 @@
- 
-#include "yacasprivate.h"
-#include "lispeval.h"
-#include "lispuserfunc.h"
-#include "standard.h"
 
-#include "lispio.h"
-#include "platfileio.h"
-#include "infixparser.h"
-#include "errors.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/lispeval.h"
+#include "yacas/lispuserfunc.h"
+#include "yacas/standard.h"
+
+#include "yacas/lispio.h"
+#include "yacas/platfileio.h"
+#include "yacas/infixparser.h"
+#include "yacas/errors.h"
+
+#include <sstream>
 
 LispUserFunction* GetUserFunction(LispEnvironment& aEnvironment,
                                   LispPtr* subList)
@@ -18,10 +20,10 @@ LispUserFunction* GetUserFunction(LispEnvironment& aEnvironment,
   {
     return userFunc;
   }
-  else if (head->String()!=NULL)
+  else if (head->String()!=nullptr)
   {
     LispMultiUserFunction* multiUserFunc = aEnvironment.MultiUserFunction(head->String());
-    if (multiUserFunc->iFileToOpen!=NULL)
+    if (multiUserFunc->iFileToOpen!=nullptr)
     {
       LispDefFile* def = multiUserFunc->iFileToOpen;
 #ifdef YACAS_DEBUG
@@ -30,31 +32,29 @@ LispUserFunction* GetUserFunction(LispEnvironment& aEnvironment,
         extern int verbose_debug;
         if (verbose_debug)
         {
-          char buf[1024];
-        #ifdef HAVE_VSNPRINTF
-          snprintf(buf,1024,"Debug> Loading file %s for function %s\n",def->iFileName->c_str(),head->String()->c_str());
-        #else
-          sprintf(buf,      "Debug> Loading file %s for function %s\n",def->iFileName->c_str(),head->String()->c_str());
-        #endif
-          aEnvironment.CurrentOutput()->Write(buf);
+            std::string msg =
+                std::string("Debug> Loading file ") +
+                std::string(def->iFileName->c_str()) +
+                std::string(" for function ") +
+                std::string(head->String()->c_str()) +
+                std::string("\n");
+            aEnvironment.CurrentOutput()->Write(msg.c_str());
         }
       }
 #endif
-      multiUserFunc->iFileToOpen=NULL;
-      InternalUse(aEnvironment,def->iFileName);
+      multiUserFunc->iFileToOpen=nullptr;
+      InternalUse(aEnvironment, def->FileName());
 
 #ifdef YACAS_DEBUG
       {
         extern int verbose_debug;
         if (verbose_debug)
         {
-          char buf[1024];
-          #ifdef HAVE_VSNPRINTF
-            snprintf(buf,1024,"Debug> Finished loading file %s\n",def->iFileName->c_str());
-          #else
-            sprintf(buf,      "Debug> Finished loading file %s\n",def->iFileName->c_str());
-          #endif
-          aEnvironment.CurrentOutput()->Write(buf);
+            std::string msg =
+                std::string("Debug> Finished loading file ") +
+                std::string(def->iFileName->c_str()) +
+                std::string("\n");
+            aEnvironment.CurrentOutput()->Write(msg.c_str());
         }
       }
 #endif
@@ -73,15 +73,7 @@ void LispEvaluatorBase::ResetStack()
 {
 }
 
-void LispEvaluatorBase::ShowStack(LispEnvironment& aEnvironment, LispOutput& aOutput)
-{
-}
-
-LispEvaluatorBase::~LispEvaluatorBase()
-{
-}
-
-BasicEvaluator::~BasicEvaluator()
+void LispEvaluatorBase::ShowStack(LispEnvironment& aEnvironment, std::ostream& aOutput)
 {
 }
 
@@ -90,24 +82,28 @@ BasicEvaluator::~BasicEvaluator()
 // be a unique (copied) element! Eg. its Nixed might be set...
 void BasicEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult, LispPtr& aExpression)
 {
-  LISPASSERT(aExpression);
+  assert(aExpression);
 
-  aEnvironment.iEvalDepth++;
-  if (aEnvironment.iEvalDepth>=aEnvironment.iMaxEvalDepth)
-  {
-    CHK2(aEnvironment.iEvalDepth<aEnvironment.iMaxEvalDepth,
-    (aEnvironment.iEvalDepth>aEnvironment.iMaxEvalDepth+20)
-    ? KLispErrUserInterrupt : KLispErrMaxRecurseDepthReached);
+  if (aEnvironment.stop_evaluation) {
+      aEnvironment.stop_evaluation = false;
+      ShowStack(aEnvironment, aEnvironment.CurrentOutput());
+      throw LispErrUserInterrupt();
   }
 
-  LispString * str = aExpression->String();
+  aEnvironment.iEvalDepth++;
+  if (aEnvironment.iEvalDepth >= aEnvironment.iMaxEvalDepth) {
+      ShowStack(aEnvironment, aEnvironment.CurrentOutput());
+      throw LispErrMaxRecurseDepthReached();
+  }
+
+  const LispString* str = aExpression->String();
 
   // Evaluate an atom: find the bound value (treat it as a variable)
   if (str)
   {
     if (str->c_str()[0] == '\"')
     {
-      aResult = (aExpression->Copy());
+      aResult = aExpression->Copy();
       goto FINISH;
     }
 
@@ -133,13 +129,11 @@ void BasicEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult, LispP
         if (head->String())
         {
           {
-            YacasEvaluator* evaluator = aEnvironment.CoreCommands().LookUp(head->String());
-            // Try to find a built-in command
-            if (evaluator)
-            {
-              evaluator->Evaluate(aResult, aEnvironment, *subList);
-              goto FINISH;
-            }
+              const auto i = aEnvironment.CoreCommands().find(head->String());
+              if (i != aEnvironment.CoreCommands().end()) {
+                  i->second.Evaluate(aResult, aEnvironment, *subList);
+                  goto FINISH;
+              }
           }
 
           {
@@ -179,14 +173,14 @@ void ShowExpression(LispString& outString, LispEnvironment& aEnvironment,
                             aEnvironment.PostFix(),
                             aEnvironment.Bodied());
   // Print out the current expression
-  StringOutput stream(outString);
-  infixprinter.Print(aExpression, stream,aEnvironment);
-
+  std::ostringstream stream;
+  infixprinter.Print(aExpression, stream, aEnvironment);
+  outString.append(stream.str());
   // Escape quotes
-  for (LispInt i = outString.Size()-1; i >= 0; --i)
+  for (LispInt i = outString.size()-1; i >= 0; --i)
   {
     if (outString[i] == '\"')
-      outString.Insert(i, LispChar('\\'));
+      outString.insert(outString.begin() + i, LispChar('\\'));
   }
 }
 
@@ -195,27 +189,27 @@ static void TraceShowExpression(LispEnvironment& aEnvironment,
 {
   LispString outString;
   ShowExpression(outString, aEnvironment, aExpression);
-  aEnvironment.CurrentOutput()->Write(&outString[0]);
+  aEnvironment.CurrentOutput().write(outString.c_str(), outString.size());
 }
 
 void TraceShowArg(LispEnvironment& aEnvironment,LispPtr& aParam,
                   LispPtr& aValue)
 {
   for (LispInt i=0;i<aEnvironment.iEvalDepth+2;i++)
-    aEnvironment.CurrentOutput()->Write("  ");
-  aEnvironment.CurrentOutput()->Write("TrArg(\"");
+    aEnvironment.CurrentOutput().write("  ", 2);
+  aEnvironment.CurrentOutput() << "TrArg(\"";
   TraceShowExpression(aEnvironment, aParam);
-  aEnvironment.CurrentOutput()->Write("\",\"");
+  aEnvironment.CurrentOutput() << "\",\"";
   TraceShowExpression(aEnvironment, aValue);
-  aEnvironment.CurrentOutput()->Write("\");\n");
+  aEnvironment.CurrentOutput() << "\");\n";
 }
 
 void TraceShowEnter(LispEnvironment& aEnvironment,
                     LispPtr& aExpression)
 {
   for (LispInt i=0;i<aEnvironment.iEvalDepth;i++)
-    aEnvironment.CurrentOutput()->Write("  ");
-  aEnvironment.CurrentOutput()->Write("TrEnter(\"");
+    aEnvironment.CurrentOutput().write("  ", 2);
+  aEnvironment.CurrentOutput() << "TrEnter(\"";
   {
     const LispChar * function = "";
     if (aExpression->SubList())
@@ -224,66 +218,57 @@ void TraceShowEnter(LispEnvironment& aEnvironment,
       if ((*sub)->String())
         function = (*sub)->String()->c_str();
     }
-    aEnvironment.CurrentOutput()->Write(function);
+    aEnvironment.CurrentOutput() << function;
   }
-  aEnvironment.CurrentOutput()->Write("\",\"");
+  aEnvironment.CurrentOutput() << "\",\"";
   TraceShowExpression(aEnvironment, aExpression);
-  aEnvironment.CurrentOutput()->Write("\",\"");
+  aEnvironment.CurrentOutput() << "\",\"";
 #ifdef YACAS_DEBUG
-  aEnvironment.CurrentOutput()->Write(
-  aExpression->iFileName ? aExpression->iFileName : ""); //file
-  aEnvironment.CurrentOutput()->Write("\",");
-  LispChar buf[30];
-  InternalIntToAscii(buf,aExpression->iLine);
-  aEnvironment.CurrentOutput()->Write(buf); //line
+  aEnvironment.CurrentOutput() << aExpression->iFileName ? aExpression->iFileName : "";
+  aEnvironment.CurrentOutput() << "\",";
+  aEnvironment.CurrentOutput() << aExpression->iLine;
 #else
-  aEnvironment.CurrentOutput()->Write(""); //file
-  aEnvironment.CurrentOutput()->Write("\",");
-  aEnvironment.CurrentOutput()->Write("0"); //line
+  aEnvironment.CurrentOutput() << ""; //file
+  aEnvironment.CurrentOutput() << "\",";
+  aEnvironment.CurrentOutput() << "0"; //line
 #endif
-  aEnvironment.CurrentOutput()->Write(");\n");
+  aEnvironment.CurrentOutput() << ");\n";
 }
 
 void TraceShowLeave(LispEnvironment& aEnvironment, LispPtr& aResult,
                     LispPtr& aExpression)
 {
   for (LispInt i=0;i<aEnvironment.iEvalDepth;i++)
-    aEnvironment.CurrentOutput()->Write("  ");
-  aEnvironment.CurrentOutput()->Write("TrLeave(\"");
+    aEnvironment.CurrentOutput().write("  ", 2);
+  aEnvironment.CurrentOutput().write("TrLeave(\"", 9);
   TraceShowExpression(aEnvironment, aExpression);
-  aEnvironment.CurrentOutput()->Write("\",\"");
+  aEnvironment.CurrentOutput().write("\",\"", 3);
   TraceShowExpression(aEnvironment, aResult);
-  aEnvironment.CurrentOutput()->Write("\");\n");
+  aEnvironment.CurrentOutput().write("\");\n", 4);
 }
 
 void TracedStackEvaluator::PushFrame()
 {
   UserStackInformation *op = NEW UserStackInformation;
-  objs.Append(op);
+  objs.push_back(op);
 }
 
 void TracedStackEvaluator::PopFrame()
 {
-  LISPASSERT (objs.Size() > 0);
-  if (objs[objs.Size()-1])
-  {
-    delete objs[objs.Size()-1];
-    objs[objs.Size()-1] = NULL;
-  }
-  objs.Delete(objs.Size()-1);
+    assert (!objs.empty());
+    delete objs.back();
+    objs.pop_back();
 }
 
 void TracedStackEvaluator::ResetStack()
 {
-  while (objs.Size()>0)
-  {
-    PopFrame();
-  }
+    while (!objs.empty())
+        PopFrame();
 }
 
 UserStackInformation& TracedStackEvaluator::StackInformation()
 {
-  return *(objs[objs.Size()-1]);
+    return *objs.back();
 }
 
 TracedStackEvaluator::~TracedStackEvaluator()
@@ -291,59 +276,54 @@ TracedStackEvaluator::~TracedStackEvaluator()
   ResetStack();
 }
 
-void TracedStackEvaluator::ShowStack(LispEnvironment& aEnvironment, LispOutput& aOutput)
+void TracedStackEvaluator::ShowStack(LispEnvironment& aEnvironment, std::ostream& aOutput)
 {
   LispLocalEvaluator local(aEnvironment,NEW BasicEvaluator);
 
   LispInt i;
   LispInt from=0;
-  LispInt upto = objs.Size();
- 
+  LispInt upto = objs.size();
+
   for (i=from;i<upto;i++)
   {
-    LispChar str[20];
 #ifdef YACAS_DEBUG
-    aEnvironment.CurrentOutput()->Write(objs[i]->iFileName);
-    aEnvironment.CurrentOutput()->Write("(");
-    InternalIntToAscii(str,objs[i]->iLine);
-    aEnvironment.CurrentOutput()->Write(str);
-    aEnvironment.CurrentOutput()->Write(") : ");
-    aEnvironment.CurrentOutput()->Write("Debug> ");
+    aEnvironment.CurrentOutput() << objs[i]->iFileName;
+    aEnvironment.CurrentOutput() << "(";
+    aEnvironment.CurrentOutput() << objs[i]->iLine;
+    aEnvironment.CurrentOutput() << ") : ";
+    aEnvironment.CurrentOutput() << "Debug> ";
 #endif
-    InternalIntToAscii(str,i);
-    aEnvironment.CurrentOutput()->Write(str);
-    aEnvironment.CurrentOutput()->Write(": ");
-    aEnvironment.CurrentPrinter().Print(objs[i]->iOperator, *aEnvironment.CurrentOutput(),aEnvironment);
+    aEnvironment.CurrentOutput() << i << ": ";
+    aEnvironment.CurrentPrinter().Print(objs[i]->iOperator, aEnvironment.CurrentOutput(),aEnvironment);
 
     LispInt internal;
-    internal = (NULL != aEnvironment.CoreCommands().LookUp(objs[i]->iOperator->String()));
+    internal =
+            aEnvironment.CoreCommands().find(objs[i]->iOperator->String()) != aEnvironment.CoreCommands().end();
     if (internal)
     {
-      aEnvironment.CurrentOutput()->Write(" (Internal function) ");
+      aEnvironment.CurrentOutput() << " (Internal function) ";
     }
     else
     {
       if (objs[i]->iRulePrecedence>=0)
       {
-        aEnvironment.CurrentOutput()->Write(" (Rule # ");
-        InternalIntToAscii(str,objs[i]->iRulePrecedence);
-        aEnvironment.CurrentOutput()->Write(str);
+        aEnvironment.CurrentOutput() << " (Rule # " << objs[i]->iRulePrecedence;
         if (objs[i]->iSide)
-          aEnvironment.CurrentOutput()->Write(" in body) ");
+          aEnvironment.CurrentOutput() << " in body) ";
         else
-          aEnvironment.CurrentOutput()->Write(" in pattern) ");
+          aEnvironment.CurrentOutput() << " in pattern) ";
       }
       else
-        aEnvironment.CurrentOutput()->Write(" (User function) ");
+        aEnvironment.CurrentOutput() << " (User function) ";
     }
     if (!!objs[i]->iExpression)
     {
-      aEnvironment.CurrentOutput()->Write("\n      ");
+      aEnvironment.CurrentOutput() << "\n      ";
       if (aEnvironment.iEvalDepth>(aEnvironment.iMaxEvalDepth-10))
       {
         LispString expr;
         PrintExpression(expr, objs[i]->iExpression,aEnvironment,60);
-        aEnvironment.CurrentOutput()->Write(expr.c_str());
+        aEnvironment.CurrentOutput() << expr;
       }
       else
       {
@@ -353,11 +333,11 @@ void TracedStackEvaluator::ShowStack(LispEnvironment& aEnvironment, LispOutput&
           LispString expr;
           LispPtr out(objs[i]->iExpression);
           PrintExpression(expr, out,aEnvironment,60);
-          aEnvironment.CurrentOutput()->Write(expr.c_str());
+          aEnvironment.CurrentOutput() << expr;
         }
       }
     }
-    aEnvironment.CurrentOutput()->Write("\n");
+    aEnvironment.CurrentOutput() << '\n';
   }
 }
 
@@ -366,13 +346,12 @@ void TracedStackEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult,
 {
   if (aEnvironment.iEvalDepth>=aEnvironment.iMaxEvalDepth)
   {
-    ShowStack(aEnvironment, *aEnvironment.CurrentOutput());
-    CHK2(aEnvironment.iEvalDepth<aEnvironment.iMaxEvalDepth,
-         KLispErrMaxRecurseDepthReached);
+      ShowStack(aEnvironment, aEnvironment.CurrentOutput());
+      throw LispErrMaxRecurseDepthReached();
   }
 
   LispPtr* subList = aExpression->SubList();
-  LispString * str = NULL;
+  const LispString* str = nullptr;
   if (subList)
   {
     LispObject* head = (*subList);
@@ -383,8 +362,8 @@ void TracedStackEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult,
       {
         PushFrame();
         UserStackInformation& st = StackInformation();
-        st.iOperator = (LispAtom::New(aEnvironment,str->c_str()));
-        st.iExpression = (aExpression);
+        st.iOperator = LispAtom::New(aEnvironment, *str);
+        st.iExpression = aExpression;
 #ifdef YACAS_DEBUG
         if (aExpression->iFileName)
         {
@@ -405,35 +384,54 @@ void TracedStackEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult,
 void TracedEvaluator::Eval(LispEnvironment& aEnvironment, LispPtr& aResult,
                            LispPtr& aExpression)
 {
-  if(!aEnvironment.iDebugger) RaiseError("Internal error: debugging failing");
-  if(aEnvironment.iDebugger->Stopped()) RaiseError("");
+  if(!aEnvironment.iDebugger)
+      throw LispErrGeneric("Internal error: debugging failing");
+  if(aEnvironment.iDebugger->Stopped())
+      throw LispErrGeneric("");
 
 REENTER:
-  errorStr.ResizeTo(1); errorStr[0] = '\0';
-  LispTrap(aEnvironment.iDebugger->Enter(aEnvironment, aExpression),errorOutput,aEnvironment);
-  if(aEnvironment.iDebugger->Stopped()) RaiseError("");
-  if (errorStr[0])
+    errorOutput.clear();
+    errorOutput.str("");
+
+  try {
+      aEnvironment.iDebugger->Enter(aEnvironment, aExpression);
+  } catch(const LispError& error) {
+      HandleError(error, aEnvironment, errorOutput);
+  }
+
+  if(aEnvironment.iDebugger->Stopped())
+      throw LispErrGeneric("");
+
+  if (!errorOutput.str().empty())
   {
-    aEnvironment.CurrentOutput()->Write(errorStr.c_str());
+    aEnvironment.CurrentOutput() << errorOutput.str();
     aEnvironment.iEvalDepth=0;
     goto REENTER;
   }
 
-  errorStr.ResizeTo(1); errorStr[0] = '\0';
-  LispTrap(BasicEvaluator::Eval(aEnvironment, aResult, aExpression),errorOutput,aEnvironment);
+    errorOutput.clear();
+    errorOutput.str("");
 
-  if (errorStr[0])
+  try {
+      BasicEvaluator::Eval(aEnvironment, aResult, aExpression);
+  } catch(const LispError& error) {
+      HandleError(error, aEnvironment, errorOutput);
+  }
+
+  if (!errorOutput.str().empty())
   {
-    aEnvironment.CurrentOutput()->Write(errorStr.c_str());
+    aEnvironment.CurrentOutput() << errorOutput.str();
     aEnvironment.iEvalDepth=0;
     aEnvironment.iDebugger->Error(aEnvironment);
     goto REENTER;
   }
 
-  if(aEnvironment.iDebugger->Stopped()) RaiseError("");
+  if(aEnvironment.iDebugger->Stopped())
+      throw LispErrGeneric("");
 
   aEnvironment.iDebugger->Leave(aEnvironment, aResult, aExpression);
-  if(aEnvironment.iDebugger->Stopped()) RaiseError("");
+  if(aEnvironment.iDebugger->Stopped())
+      throw LispErrGeneric("");
 }
 
 YacasDebuggerBase::~YacasDebuggerBase()
@@ -467,7 +465,7 @@ void DefaultDebugger::Leave(LispEnvironment& aEnvironment, LispPtr& aResult,
   defaultEval.Eval(aEnvironment, result, iLeave);
 }
 
-LispBoolean DefaultDebugger::Stopped()
+bool DefaultDebugger::Stopped()
 {
   return iStopped;
 }
