@@ -1,17 +1,14 @@
 
 
-#include "yacasprivate.h"
-#include "lispevalhash.h"
-#include "lispenvironment.h"
-#include "lispatom.h"
-#include "lispeval.h"
-#include "errors.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/lispevalhash.h"
+#include "yacas/lispenvironment.h"
+#include "yacas/lispatom.h"
+#include "yacas/lispeval.h"
+#include "yacas/errors.h"
 
 
 
-#define InternalEval aEnvironment.iEvaluator->Eval
-
-
 void YacasEvaluator::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,LispPtr& aArguments)
 {
 
@@ -38,7 +35,9 @@ void YacasEvaluator::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,Lis
   {
     for (i=0;i<nr;i++)
     {
-      Check(iter.getObj(), KLispErrWrongNumberOfArgs);
+      if (!iter.getObj())
+          throw LispErrWrongNumberOfArgs();
+
       aEnvironment.iStack.PushArgOnStack(iter.getObj()->Copy());
       ++iter;
     }
@@ -54,8 +53,10 @@ void YacasEvaluator::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,Lis
     LispPtr arg;
     for (i=0;i<nr;i++)
     {
-      Check(iter.getObj(), KLispErrWrongNumberOfArgs);
-      InternalEval(aEnvironment, arg, *iter);
+      if (!iter.getObj())
+          throw LispErrWrongNumberOfArgs();
+
+      aEnvironment.iEvaluator->Eval(aEnvironment, arg, *iter);
       aEnvironment.iStack.PushArgOnStack(arg);
       ++iter;
     }
@@ -75,7 +76,7 @@ PrintExpression(res, list,aEnvironment,100);
 printf("before %s\n",res.String());
 */
 
-      InternalEval(aEnvironment, arg, list);
+      aEnvironment.iEvaluator->Eval(aEnvironment, arg, list);
 
 /*
 PrintExpression(res, arg,aEnvironment,100);
