@@ -1,6 +1,6 @@
 
-#include "yacasprivate.h"
-#include "lispobject.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/lispobject.h"
 
 #ifdef YACAS_DEBUG
 long theNrCurrent=0;
@@ -32,7 +32,7 @@ LispInt LispObject::Equal(LispObject& aOther)
     //So, no strings.
     LispPtr *iter1 = SubList();
     LispPtr *iter2 = aOther.SubList();
-    LISPASSERT(!!iter1 && !!iter2);
+    assert(!!iter1 && !!iter2);
 
     // check all elements in sublist
     while (!!(*iter1) && !!(*iter2))
