@@ -1,35 +1,31 @@
 
-#include "yacasprivate.h"
+#include "yacas/yacasprivate.h"
 
-#include "yacasbase.h"
-#include "lispenvironment.h"
-#include "standard.h"
-#include "lispeval.h"
-#include "lispatom.h"
-#include "lispparser.h"
-#include "stdfileio.h"
-#include "stringio.h"
-#include "lisperror.h"
-#include "infixparser.h"
-#include "lispuserfunc.h"
-#include "mathuserfunc.h"
-#include "platmath.h"
-#include "numbers.h"
-#include "arrayclass.h"
-#include "patternclass.h"
-#include "substitute.h"
-#include "errors.h"
-#include "patcher.h"
+#include "yacas/yacasbase.h"
+#include "yacas/lispenvironment.h"
+#include "yacas/standard.h"
+#include "yacas/lispeval.h"
+#include "yacas/lispatom.h"
+#include "yacas/lispparser.h"
+#include "yacas/stdfileio.h"
+#include "yacas/stringio.h"
+#include "yacas/lisperror.h"
+#include "yacas/infixparser.h"
+#include "yacas/lispuserfunc.h"
+#include "yacas/mathuserfunc.h"
+#include "yacas/platmath.h"
+#include "yacas/numbers.h"
+#include "yacas/arrayclass.h"
+#include "yacas/patternclass.h"
+#include "yacas/substitute.h"
+#include "yacas/errors.h"
+#include "yacas/patcher.h"
 
-#ifdef HAVE_MATH_H
-  #include <math.h>
-#endif
-
-#undef VERSION
-
-#include "version.h"
+#include <cmath>
 
+#include "yacas/yacas_version.h"
 
+#include <sstream>
 
 
 #define InternalEval aEnvironment.iEvaluator->Eval
@@ -47,13 +43,13 @@
 void GetNumber(RefPtr<BigNumber>& x, LispEnvironment& aEnvironment, LispInt aStackTop, LispInt aArgNr)
 {
     x = ARGUMENT(aArgNr)->Number(aEnvironment.Precision());
-    CHK_ARG_CORE(x,aArgNr);  // was: x.Ptr()
+    CheckArg(x, aArgNr, aEnvironment, aStackTop);
 }
 
 //FIXME remove these
 void LispArithmetic2(LispEnvironment& aEnvironment, LispInt aStackTop,
                      LispObject* (*func)(LispObject* f1, LispObject* f2,LispEnvironment& aEnvironment,LispInt aPrecision),
-                    LispBoolean arbbase=LispFalse);
+                    bool arbbase=false);
 
 void LispArithmetic1(LispEnvironment& aEnvironment, LispInt aStackTop,
                      LispObject* (*func)(LispObject* f1, LispEnvironment& aEnvironment,LispInt aPrecision));
@@ -67,7 +63,7 @@ void LispArithmetic1(LispEnvironment& aEnvironment, LispInt aStackTop,
 void LispArithmetic1(LispEnvironment& aEnvironment, LispInt aStackTop,
                      LispObject* (*func)(LispObject* f1, LispEnvironment& aEnvironment,LispInt aPrecision))
 {
-    CHK_ARG_CORE(ARGUMENT(1)->Number(0) != NULL,1);
+    CheckArg(ARGUMENT(1)->Number(0), 1, aEnvironment, aStackTop);
     RESULT = (func(ARGUMENT(1), aEnvironment, aEnvironment.Precision()));
 }
 
@@ -75,12 +71,12 @@ void LispArithmetic1(LispEnvironment& aEnvironment, LispInt aStackTop,
 //FIXME remove these
 void LispArithmetic2(LispEnvironment& aEnvironment, LispInt aStackTop,
                      LispObject* (*func)(LispObject* f1, LispObject* f2,LispEnvironment& aEnvironment,LispInt aPrecision),
-                    LispBoolean arbbase)
+                    bool arbbase)
 {
     if (!arbbase)
     {
-        CHK_ARG_CORE(ARGUMENT(1)->Number(0)  != NULL,1);
-        CHK_ARG_CORE(ARGUMENT(2)->Number(0)  != NULL,2);
+        CheckArg(ARGUMENT(1)->Number(0), 1, aEnvironment, aStackTop);
+        CheckArg(ARGUMENT(2)->Number(0), 2, aEnvironment, aStackTop);
     }
     RESULT = (func(ARGUMENT(1),ARGUMENT(2),
                                    aEnvironment,
@@ -111,8 +107,9 @@ void LispMultiply(LispEnvironment& aEnvironment, LispInt aStackTop)
 //TODO we need to have Gcd in BigNumber!
 void LispGcd(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    CHK_ARG_CORE(ARGUMENT(1)->Number(0)  != NULL,1);
-    CHK_ARG_CORE(ARGUMENT(1)->Number(0)  != NULL,2);
+    CheckArg(ARGUMENT(1)->Number(0), 1, aEnvironment, aStackTop);
+    CheckArg(ARGUMENT(2)->Number(0), 2, aEnvironment, aStackTop);
+
     RESULT = (GcdInteger(ARGUMENT(1),ARGUMENT(2),aEnvironment));
 }
 
@@ -386,49 +383,35 @@ void LispFastIsPrime(LispEnvironment& aEnvironment, LispInt aStackTop)
 
 // define a macro to replace all platform math functions
 
-  #ifndef HAVE_MATH_H
-  // this warning is here just to be sure what we are compiling
-    #warning do not have math.h
-    #define PLATFORM_UNARY(LispName, PlatformName, LispBackupName, OldName) \
-void LispName(LispEnvironment& aEnvironment, LispInt aStackTop) \
-{ \
-      LispBackupName(aEnvironment, aStackTop); \
-}
-    #define PLATFORM_BINARY(LispName, PlatformName, LispBackupName, OldName) \
-void LispName(LispEnvironment& aEnvironment, LispInt aStackTop) \
-{ \
-      LispBackupName(aEnvironment, aStackTop); \
-}
-  #else  // HAVE_MATH_H
-    #define PLATFORM_UNARY(LispName, PlatformName, LispBackupName, OldName) \
-void LispName(LispEnvironment& aEnvironment, LispInt aStackTop) \
-{ \
-      RefPtr<BigNumber> x; \
-      GetNumber(x,aEnvironment, aStackTop, 1); \
-      double result = PlatformName(x->Double()); \
-      BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision()); \
-      z->SetTo(result); \
-      RESULT = (NEW LispNumber(z)); \
-}
-    #define PLATFORM_BINARY(LispName, PlatformName, LispBackupName, OldName) \
-void LispName(LispEnvironment& aEnvironment, LispInt aStackTop) \
-{ \
-      RefPtr<BigNumber> x, y; \
-      GetNumber(x,aEnvironment, aStackTop, 1); \
-      GetNumber(y,aEnvironment, aStackTop, 2); \
-      double result = PlatformName(x->Double(), y->Double()); \
-      BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision()); \
-      z->SetTo(result); \
-      RESULT = (NEW LispNumber(z)); \
-}
-  #endif
+#define PLATFORM_UNARY(LispName, PlatformName, LispBackupName, OldName) \
+    void LispName(LispEnvironment& aEnvironment, LispInt aStackTop)     \
+    {                                                                   \
+        RefPtr<BigNumber> x;                                            \
+        GetNumber(x,aEnvironment, aStackTop, 1);                        \
+        double result = PlatformName(x->Double());                      \
+        BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision());   \
+        z->SetTo(result);                                               \
+        RESULT = (NEW LispNumber(z));                                   \
+    }
+
+#define PLATFORM_BINARY(LispName, PlatformName, LispBackupName, OldName) \
+    void LispName(LispEnvironment& aEnvironment, LispInt aStackTop)     \
+    {                                                                   \
+        RefPtr<BigNumber> x, y;                                         \
+        GetNumber(x,aEnvironment, aStackTop, 1);                        \
+        GetNumber(y,aEnvironment, aStackTop, 2);                        \
+        double result = PlatformName(x->Double(), y->Double());         \
+        BigNumber *z = NEW BigNumber(aEnvironment.BinaryPrecision());   \
+        z->SetTo(result);                                               \
+        RESULT = (NEW LispNumber(z));                                   \
+    }
 
 // now we can define all such functions, e.g.:
 
 // some or all of these functions should be moved to scripts
-  PLATFORM_UNARY(LispFastArcSin, asin, LispArcSin, PlatArcSin)
-  PLATFORM_UNARY(LispFastLog, log, LispLn, PlatLn)
-  PLATFORM_BINARY(LispFastPower, pow, LispPower, PlatPower)
+PLATFORM_UNARY(LispFastArcSin, std::asin, LispArcSin, PlatArcSin)
+PLATFORM_UNARY(LispFastLog, std::log, LispLn, PlatLn)
+PLATFORM_BINARY(LispFastPower, std::pow, LispPower, PlatPower)
 
 
 
@@ -460,22 +443,22 @@ void LispFromBase(LispEnvironment& aEnvironment, LispInt aStackTop)
     LispPtr oper(ARGUMENT(1));
     // Check that result is a number, and that it is in fact an integer
     RefPtr<BigNumber> num; num = oper->Number(aEnvironment.BinaryPrecision());
-    CHK_ARG_CORE(num,1);  // was: num.Ptr()
+    CheckArg(num, 1, aEnvironment, aStackTop);
   // check that the base is an integer between 2 and 32
-    CHK_ARG_CORE(num->IsInt()
-        && num->Double() >= BASE2 && num->Double() <= log2_table_range(), 1);
+    CheckArg(num->IsInt() && num->Double() >= BASE2 && num->Double() <= log2_table_range(), 1, aEnvironment, aStackTop);
 
     // Get a short platform integer from the first argument
     LispInt base = (LispInt)(num->Double());
 
     // Get the number to convert
     LispPtr fromNum(ARGUMENT(2));
-    LispString * str2 = fromNum->String();
-    CHK_ARG_CORE(str2,2);
+    const LispString* str2 = fromNum->String();
+    CheckArg(str2, 2, aEnvironment, aStackTop);
 
     // Added, unquote a string
-    CHK_ARG_CORE(InternalIsString(str2),2);
-    str2 = aEnvironment.HashTable().LookUpUnStringify(str2->c_str());
+    CheckArg(InternalIsString(str2), 2, aEnvironment, aStackTop);
+    //str2 = aEnvironment.HashTable().LookUpCounted(str2->c_str() + 1, std::strlen(str2->c_str()) - 2);
+    str2 = aEnvironment.HashTable().LookUp(str2->substr(1, str2->length() - 2));
 
     // convert using correct base
   // FIXME: API breach, must pass precision in base digits and not in bits!
@@ -491,10 +474,9 @@ void LispToBase(LispEnvironment& aEnvironment, LispInt aStackTop)
     LispPtr oper(ARGUMENT(1));
     // Check that result is a number, and that it is in fact an integer
     RefPtr<BigNumber> num; num = oper->Number(aEnvironment.BinaryPrecision());
-    CHK_ARG_CORE(num,1);
+    CheckArg(num, 1, aEnvironment, aStackTop);
   // check that the base is an integer between 2 and 32
-    CHK_ARG_CORE(num->IsInt()
-        && num->Double() >= BASE2 && num->Double() <= log2_table_range(), 1);
+    CheckArg(num->IsInt() && num->Double() >= BASE2 && num->Double() <= log2_table_range(), 1, aEnvironment, aStackTop);
 
     // Get a short platform integer from the first argument
     LispInt base = (LispInt)(num->Double());
@@ -511,7 +493,7 @@ void LispToBase(LispEnvironment& aEnvironment, LispInt aStackTop)
     x->ToString(str,aEnvironment.BinaryPrecision(),base);
     // Get unique string from hash table, and create an atom from it.
 
-    RESULT = (LispAtom::New(aEnvironment,aEnvironment.HashTable().LookUpStringify(str.c_str())->c_str()));
+    RESULT = LispAtom::New(aEnvironment, Stringify(str));
 }
 
 
@@ -521,10 +503,10 @@ void LispApplyPure(LispEnvironment& aEnvironment, LispInt aStackTop)
     LispPtr oper(ARGUMENT(1));
     LispPtr args(ARGUMENT(2));
 
-    CHK_ARG_CORE(args->SubList(),2);
-    CHK_CORE((*args->SubList()),2);
+    CheckArg(args->SubList(), 2, aEnvironment, aStackTop);
+    CheckArg(*args->SubList(), 2, aEnvironment, aStackTop);
+
 
- 
 
     // Apply a pure string
     if (oper->String())
@@ -536,8 +518,8 @@ void LispApplyPure(LispEnvironment& aEnvironment, LispInt aStackTop)
     else
     {   // Apply a pure function {args,body}.
         LispPtr args2((*args->SubList())->Nixed());
-        CHK_ARG_CORE(oper->SubList(),1);
-        CHK_ARG_CORE((*oper->SubList()),1);
+        CheckArg(oper->SubList(), 1, aEnvironment, aStackTop);
+        CheckArg(*oper->SubList(), 1, aEnvironment, aStackTop);
         InternalApplyPure(oper,args2,RESULT,aEnvironment);
     }
 }
@@ -549,14 +531,14 @@ void YacasPrettyReaderSet(LispEnvironment& aEnvironment, LispInt aStackTop)
 
   if (nrArguments == 1)
   {
-    aEnvironment.SetPrettyReader(NULL);
+    aEnvironment.SetPrettyReader(nullptr);
   }
   else
   {
-    CHK_CORE(nrArguments == 2,KLispErrWrongNumberOfArgs);
+    CheckNrArgs(2, ARGUMENT(0), aEnvironment);
     LispPtr oper(ARGUMENT(0));
     oper = oper->Nixed(); // oper.GoNext();  // woof woof woof
-    CHK_ISSTRING_CORE(oper,1);
+    CheckArgIsString(oper, 1, aEnvironment, aStackTop);
     aEnvironment.SetPrettyReader(oper->String());
   }
   InternalTrue(aEnvironment,RESULT);
@@ -565,9 +547,9 @@ void YacasPrettyReaderSet(LispEnvironment& aEnvironment, LispInt aStackTop)
 void YacasPrettyReaderGet(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
   if (!aEnvironment.PrettyReader())
-    RESULT = (LispAtom::New(aEnvironment,"\"\""));
+    RESULT = LispAtom::New(aEnvironment, "\"\"");
   else
-    RESULT = (LispAtom::New(aEnvironment,aEnvironment.PrettyReader()->c_str()));
+    RESULT = LispAtom::New(aEnvironment, *aEnvironment.PrettyReader());
 }
 
 void YacasPrettyPrinterSet(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -576,14 +558,14 @@ void YacasPrettyPrinterSet(LispEnvironment& aEnvironment, LispInt aStackTop)
 
   if (nrArguments == 1)
   {
-    aEnvironment.SetPrettyPrinter(NULL);
+    aEnvironment.SetPrettyPrinter(nullptr);
   }
   else
   {
-    CHK_CORE(nrArguments == 2,KLispErrWrongNumberOfArgs);
+    CheckNrArgs(2, ARGUMENT(0), aEnvironment);
     LispPtr oper(ARGUMENT(0));
     oper = oper->Nixed(); // oper.GoNext();  // woof woof woof
-    CHK_ISSTRING_CORE(oper,1);
+    CheckArgIsString(oper, 1, aEnvironment, aStackTop);
     aEnvironment.SetPrettyPrinter(oper->String());
   }
   InternalTrue(aEnvironment,RESULT);
@@ -592,9 +574,9 @@ void YacasPrettyPrinterSet(LispEnvironment& aEnvironment, LispInt aStackTop)
 void YacasPrettyPrinterGet(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
   if (!aEnvironment.PrettyPrinter())
-    RESULT = (LispAtom::New(aEnvironment,"\"\""));
+    RESULT = LispAtom::New(aEnvironment, "\"\"");
   else
-    RESULT = (LispAtom::New(aEnvironment,aEnvironment.PrettyPrinter()->c_str()));
+    RESULT = LispAtom::New(aEnvironment, *aEnvironment.PrettyPrinter());
 }
 
 void LispGarbageCollect(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -606,19 +588,22 @@ void LispGarbageCollect(LispEnvironment& aEnvironment, LispInt aStackTop)
 void LispPatchLoad(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
   LispPtr evaluated(ARGUMENT(1));
-  LispString * string = evaluated->String();
-  CHK_ARG_CORE(string, 1);
+  const LispString* string = evaluated->String();
+  CheckArg(string, 1, aEnvironment, aStackTop);
   LispString oper;
-  InternalUnstringify(oper, string);
-  LispString * hashedname = aEnvironment.HashTable().LookUp(oper.c_str());
+  InternalUnstringify(oper, *string);
+  const LispString* hashedname = aEnvironment.HashTable().LookUp(oper);
   InputStatus oldstatus = aEnvironment.iInputStatus;
   aEnvironment.iInputStatus.SetTo(hashedname->c_str());
-  LispLocalFile localFP(aEnvironment, oper.c_str(), LispTrue,
+  LispLocalFile localFP(aEnvironment, oper, true,
                         aEnvironment.iInputDirectories);
-  Check(localFP.iOpened != 0, KLispErrFileNotFound);
-  FILEINPUT newInput(localFP,aEnvironment.iInputStatus);
+
+  if (!localFP.stream.is_open())
+      throw LispErrFileNotFound();
+
+  CachedStdFileInput newInput(localFP,aEnvironment.iInputStatus);
   PatchLoad(newInput.StartPtr(),
-            *aEnvironment.CurrentOutput(),
+            aEnvironment.CurrentOutput(),
             aEnvironment);
   aEnvironment.iInputStatus.RestoreFrom(oldstatus);
   InternalTrue(aEnvironment,RESULT);
@@ -627,15 +612,15 @@ void LispPatchLoad(LispEnvironment& aEnvironment, LispInt aStackTop)
 void LispPatchString(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
   LispPtr evaluated(ARGUMENT(1));
-  LispString * string = evaluated->String();
-  CHK_ARG_CORE(string, 1);
+  const LispString* string = evaluated->String();
+  CheckArg(string, 1, aEnvironment, aStackTop);
   LispString oper;
-  InternalUnstringify(oper, string);
-  LispString str;
-  StringOutput newOutput(str);
-  LispLocalOutput localOutput(aEnvironment, &newOutput);
-  PatchLoad(oper.c_str(), newOutput, aEnvironment);
-  RESULT = (LispAtom::New(aEnvironment,aEnvironment.HashTable().LookUpStringify(str.c_str())->c_str()));
+  InternalUnstringify(oper, *string);
+
+  std::ostringstream os;
+  LispLocalOutput localOutput(aEnvironment, os);
+  PatchLoad(oper.c_str(), os, aEnvironment);
+  RESULT = LispAtom::New(aEnvironment, Stringify(os.str()));
 }
 
 void YacasExtraInfoSet(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -683,9 +668,9 @@ void LispXmlTokenizer(LispEnvironment& aEnvironment, LispInt aStackTop)
 void LispExplodeTag(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
   LispPtr out(ARGUMENT(1));
-  CHK_ISSTRING_CORE(out,1);
+  CheckArgIsString(1, aEnvironment, aStackTop);
 
-  LispChar * str = out->String()->c_str();
+  const LispChar* str = out->String()->c_str();
   str++;
   if (str[0] != '<')
   {
@@ -694,68 +679,64 @@ void LispExplodeTag(LispEnvironment& aEnvironment, LispInt aStackTop)
   }
   str++;
   const LispChar * type = (str[0] == '/') ? (str++, "\"Close\"") : "\"Open\"";
-  LispString tag;
-  tag.ResizeTo(0);
+  std::string tag;
 
-  tag.Append('\"');
+  tag.push_back('\"');
   while (IsAlpha(*str))
   {
     LispChar c = *str++;
     if (c >= 'a' && c <= 'z')
       c = c + ('A'-'a');
-    tag.Append(c);
+    tag.push_back(c);
   }
-  tag.Append('\"');
-  tag.Append('\0');
+  tag.push_back('\"');
 
-  LispObject* info = NULL;
+  LispObject* info = nullptr;
 
   while (*str == ' ') str++;
   while (*str != '>' && *str != '/')
   {
-    LispString name;
-    name.ResizeTo(0);
-    name.Append('\"');
+    std::string name;
+    name.push_back('\"');
 
     while (IsAlpha(*str))
     {
       LispChar c = *str++;
       if (c >= 'a' && c <= 'z')
         c = c + ('A'-'a');
-      name.Append(c);
+      name.push_back(c);
     }
-    name.Append('\"');
-    name.Append('\0');
-    CHK_ARG_CORE(str[0] == '=',1);
-    str++;
-    CHK_ARG_CORE(str[0] == '\"',1);
-    LispString value;
-    value.ResizeTo(0);
-    value.Append(*str++);
-    while (*str != '\"')
-    {
-      value.Append(*str++);
-    }
-    value.Append(*str++);
-    value.Append('\0');
+    name.push_back('\"');
 
-    info =  LIST(LA(ATOML("List")) + LA(ATOML(name.c_str())) + LA(ATOML(value.c_str()))) + LA(info);
+    CheckArg(str[0] == '=', 1, aEnvironment, aStackTop);
+    str++;
+    CheckArg(str[0] == '\"', 1, aEnvironment, aStackTop);
+
+    std::string value;
+    value.push_back(*str++);
+    while (*str != '\"')
+      value.push_back(*str++);
+
+    value.push_back(*str++);
+
+    info =  LispSubList::New(LispObjectAdder(aEnvironment.iList->Copy()) + LispObjectAdder(LispAtom::New(aEnvironment, name)) + LispObjectAdder(LispAtom::New(aEnvironment, value))) + LispObjectAdder(info);
     while (*str == ' ') str++;
   }
   if (*str == '/')
   {
     type = "\"OpenClose\"";
     str++;
-    while (*str == ' ') str++;
+    while (*str == ' ')
+        ++str;
   }
- 
-  info = LIST(LA(ATOML("List")) + LA(info));
+
+  info = LispSubList::New(LispObjectAdder(aEnvironment.iList->Copy()) + LispObjectAdder(info));
   RESULT = (
-              LIST(
-                   LA(ATOML("XmlTag")) +
-                   LA(ATOML(tag.c_str())) +
-                   LA(info) +
-                   LA(ATOML(type))
+              LispSubList::New(
+                   LispObjectAdder(LispAtom::New(aEnvironment, "XmlTag")) +
+                   LispObjectAdder(LispAtom::New(aEnvironment, tag)) +
+                   LispObjectAdder(info) +
+                   LispObjectAdder(LispAtom::New(aEnvironment, type))
                   )
              );
 }
@@ -767,16 +748,16 @@ void YacasBuiltinAssoc(LispEnvironment& aEnvironment, LispInt aStackTop)
 
     // key to find
     LispPtr key(ARGUMENT(1));
- 
+
     // assoc-list to find it in
     LispPtr list(ARGUMENT(2));
 
     LispObject* t;
 
     //Check that it is a compound object
-    CHK_ARG_CORE(list->SubList(), 2);
+    CheckArg(list->SubList(), 2, aEnvironment, aStackTop);
     t = (*list->SubList());
-    CHK_ARG_CORE(t, 2);
+    CheckArg(t, 2, aEnvironment, aStackTop);
     t = t->Nixed();
 
     while (t)
@@ -793,26 +774,23 @@ void YacasBuiltinAssoc(LispEnvironment& aEnvironment, LispInt aStackTop)
                     RESULT = (t);
                     return;
                 }
- 
+
             }
         }
         t = t->Nixed();
     }
 
- 
-    RESULT = (ATOML("Empty"));
+    RESULT = (LispAtom::New(aEnvironment, "Empty"));
 }
 
 void LispCurrentFile(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    RESULT = (LispAtom::New(aEnvironment,aEnvironment.HashTable().LookUpStringify(aEnvironment.iInputStatus.FileName())->c_str()));
+    RESULT = LispAtom::New(aEnvironment, Stringify(aEnvironment.iInputStatus.FileName()));
 }
 
 void LispCurrentLine(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    LispChar s[30];
-    InternalIntToAscii(s, aEnvironment.iInputStatus.LineNumber());
-    RESULT = (LispAtom::New(aEnvironment,s));
+    RESULT = LispAtom::New(aEnvironment, std::to_string(aEnvironment.iInputStatus.LineNumber()));
 }
 
 void LispBackQuote(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -823,10 +801,9 @@ void LispBackQuote(LispEnvironment& aEnvironment, LispInt aStackTop)
     InternalEval(aEnvironment, RESULT, result);
 }
 
-// this function is declared in yacasapi.cpp
 void LispVersion(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    RESULT = (LispAtom::New(aEnvironment,"\"" VERSION "\""));
+    RESULT = (LispAtom::New(aEnvironment,"\"" YACAS_VERSION "\""));
 }
 
 
@@ -846,7 +823,9 @@ void LispBitsToDigits(LispEnvironment& aEnvironment, LispInt aStackTop)
   }
   else
   {
-    RaiseError("BitsToDigits: error: arguments (%f, %f) must be small integers", x->Double(), y->Double());
+      std::ostringstream buf;
+      buf << "BitsToDigits: error: arguments (" << x->Double() << ", " << y->Double() << " must be small integers";
+      throw LispErrGeneric(buf.str());
   }
   BigNumber *z = NEW BigNumber();
   z->SetTo(result);
@@ -869,10 +848,11 @@ void LispDigitsToBits(LispEnvironment& aEnvironment, LispInt aStackTop)
   }
   else
   {
-    RaiseError("DigitsToBits: error: arguments (%f, %f) must be small integers", x->Double(), y->Double());
+      std::ostringstream buf;
+      buf << "BitsToDigits: error: arguments (" << x->Double() << ", " << y->Double() << " must be small integers";
+      throw LispErrGeneric(buf.str());
   }
   BigNumber *z = NEW BigNumber();
   z->SetTo(result);
   RESULT = (NEW LispNumber(z));
 }
-
