@@ -1,9 +1,22 @@
 #include <assert.h>
+#include <string.h>
 #include <stdint.h>
 
+#ifndef WIN32
+#include <pthread.h>
+#else
+#include <windows.h>
+#endif
+
+
+#ifdef WIN32
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
+#endif
+
 #include <sys/types.h>
 
-#include "yacasprivate.h"
+#include "yacas/yacasprivate.h"
 
 #define WITH_PYMALLOC
 
@@ -234,11 +247,24 @@ static int running_on_valgrind = -1;
 /*
  * Python's threads are serialized, so object malloc locking is disabled.
  */
-#define SIMPLELOCK_DECL(lock)   /* simple lock declaration              */
-#define SIMPLELOCK_INIT(lock)   /* allocate (if needed) and initialize  */
-#define SIMPLELOCK_FINI(lock)   /* free/destroy an existing lock        */
-#define SIMPLELOCK_LOCK(lock)   /* acquire released lock */
-#define SIMPLELOCK_UNLOCK(lock) /* release acquired lock */
+
+namespace {
+    static bool _yacas_threadsafe_malloc = false;
+}
+
+#ifdef WIN32
+#define SIMPLELOCK_DECL(lock) CRITICAL_SECTION lock;
+#define SIMPLELOCK_INIT(lock) { if (_yacas_threadsafe_malloc) InitializeCriticalSectionAndSpinCount(&lock, 0x00000400); }
+#define SIMPLELOCK_FINI(lock) { if (_yacas_threadsafe_malloc) DeleteCriticalSection(&lock); }
+#define SIMPLELOCK_LOCK(lock) { if (_yacas_threadsafe_malloc) EnterCriticalSection(&lock); }
+#define SIMPLELOCK_UNLOCK(lock) { if (_yacas_threadsafe_malloc) LeaveCriticalSection(&lock); }
+#else
+#define SIMPLELOCK_DECL(lock) pthread_mutex_t lock;
+#define SIMPLELOCK_INIT(lock) { if (_yacas_threadsafe_malloc) pthread_mutex_init(&lock); }
+#define SIMPLELOCK_FINI(lock) { if (_yacas_threadsafe_malloc) pthread_mutex_destroy(&lock); }
+#define SIMPLELOCK_LOCK(lock) { if (_yacas_threadsafe_malloc) pthread_mutex_lock(&lock); }
+#define SIMPLELOCK_UNLOCK(lock) { if (_yacas_threadsafe_malloc) pthread_mutex_unlock(&lock); }
+#endif
 
 /*
  * Basic types
@@ -1959,3 +1985,7 @@ void *PlatObReAlloc(void *p, size_t nbytes)
     return PyObject_Realloc(p, nbytes);
 }
 
+void PlatObSetThreadSafe(bool ts)
+{
+    _yacas_threadsafe_malloc = ts;
+}
