@@ -1,6 +1,6 @@
 
-#include "yacasprivate.h"
-#include "patternclass.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/patternclass.h"
 
 PatternClass::PatternClass(YacasPatternPredicateBase* aPatternMatcher)
 : iPatternMatcher(aPatternMatcher)
@@ -12,30 +12,25 @@ PatternClass::~PatternClass()
     delete iPatternMatcher;
 }
 
-const LispChar * PatternClass::Send(LispArgList& aArgList)
-{
-    return NULL;
-}
-
 const LispChar * PatternClass::TypeName()
 {
     return "\"Pattern\"";
 }
 
-LispBoolean PatternClass::Matches(LispEnvironment& aEnvironment,
+bool PatternClass::Matches(LispEnvironment& aEnvironment,
                                   LispPtr& aArguments)
 {
-    LISPASSERT(iPatternMatcher);
-    LispBoolean result;
+    assert(iPatternMatcher);
+    bool result;
     result = iPatternMatcher->Matches(aEnvironment, aArguments);
     return result;
 }
 
-LispBoolean PatternClass::Matches(LispEnvironment& aEnvironment,
+bool PatternClass::Matches(LispEnvironment& aEnvironment,
                                   LispPtr* aArguments)
 {
-    LISPASSERT(iPatternMatcher);
-    LispBoolean result;
+    assert(iPatternMatcher);
+    bool result;
     result = iPatternMatcher->Matches(aEnvironment, aArguments);
     return result;
 }
