@@ -1,171 +1,119 @@
-
-
-
-#include "yacasprivate.h"
-#include "yacasbase.h"
-#include "patterns.h"
-#include "standard.h"
-#include "mathuserfunc.h"
-#include "lispobject.h"
-#include "lispeval.h"
-#include "standard.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/yacasbase.h"
+#include "yacas/patterns.h"
+#include "yacas/standard.h"
+#include "yacas/mathuserfunc.h"
+#include "yacas/lispobject.h"
+#include "yacas/lispeval.h"
+#include "yacas/standard.h"
 
 #ifdef YACAS_DEBUG
 #include <stdio.h>
 #endif // YACAS_DEBUG
 
-
-#define InternalEval aEnvironment.iEvaluator->Eval
-
-
-YacasParamMatcherBase::~YacasParamMatcherBase()
-{
-}
-
-
-MatchAtom::MatchAtom(LispString * aString) : iString(aString)
-{
-}
-
-LispBoolean MatchAtom::ArgumentMatches(LispEnvironment& aEnvironment,
+bool MatchAtom::ArgumentMatches(LispEnvironment& aEnvironment,
                                        LispPtr& aExpression,
-                                       LispPtr* arguments)
+                                       LispPtr* arguments) const
 {
     // If it is a floating point, don't even bother comparing
     if (!!aExpression)
       if (aExpression->Number(0))
         if (!aExpression->Number(0)->IsInt())
-          return LispFalse;
- 
+          return false;
+
     return (iString == aExpression->String());
 }
 
 
-
-
-
-
-
-MatchNumber::MatchNumber(BigNumber* aNumber) : iNumber(aNumber)
-{
-}
-
-LispBoolean MatchNumber::ArgumentMatches(LispEnvironment& aEnvironment,
+bool MatchNumber::ArgumentMatches(LispEnvironment& aEnvironment,
                                        LispPtr& aExpression,
-                                       LispPtr* arguments)
+                                       LispPtr* arguments) const
 {
-  if (aExpression->Number(aEnvironment.Precision()))
-    return iNumber->Equals(*aExpression->Number(aEnvironment.Precision()));
-  return LispFalse;
+    if (aExpression->Number(aEnvironment.Precision()))
+        return iNumber->Equals(*aExpression->Number(aEnvironment.Precision()));
+
+    return false;
 }
 
-
-
-
-MatchVariable::MatchVariable(LispInt aVarIndex) : iVarIndex(aVarIndex)
-{
-}
-LispBoolean MatchVariable::ArgumentMatches(LispEnvironment& aEnvironment,
+bool MatchVariable::ArgumentMatches(LispEnvironment& aEnvironment,
                                        LispPtr& aExpression,
-                                       LispPtr* arguments)
+                                       LispPtr* arguments) const
 {
-    if (!arguments[iVarIndex])
-    {
-        arguments[iVarIndex] = (aExpression);
-        return LispTrue;
+    if (!arguments[iVarIndex]) {
+        arguments[iVarIndex] = aExpression;
+        return true;
     }
-    else
-    {
-        if (InternalEquals(aEnvironment, aExpression, arguments[iVarIndex]))
-        {
-            return LispTrue;
-        }
-        return LispFalse;
-    }
-    return LispFalse;
+
+    if (InternalEquals(aEnvironment, aExpression, arguments[iVarIndex]))
+        return true;
+
+    return false;
 }
 
-MatchSubList::MatchSubList(YacasParamMatcherBase** aMatchers,
-                           LispInt aNrMatchers)
-: iMatchers(aMatchers),iNrMatchers(aNrMatchers)
-
-{
-}
-
-MatchSubList::~MatchSubList()
-{
-    if (iMatchers)
-    {
-        LispInt i;
-        for (i=0;i<iNrMatchers;i++)
-            delete iMatchers[i];
-        PlatFree(iMatchers);
-    }
-}
-
-LispBoolean MatchSubList::ArgumentMatches(LispEnvironment& aEnvironment,
+bool MatchSubList::ArgumentMatches(LispEnvironment& aEnvironment,
                                           LispPtr& aExpression,
-                                          LispPtr* arguments)
+                                          LispPtr* arguments) const
 {
   if (!aExpression->SubList())
-    return LispFalse;
+    return false;
 
   LispIterator iter(aExpression);
   LispObject * pObj = iter.getObj();
-  Check(pObj,KLispErrInvalidArg);
+
+  if (!pObj)
+      throw LispErrInvalidArg();
+
   LispPtr * pPtr = pObj->SubList();
-  Check(pPtr,KLispErrNotList);
+
+  if (!pPtr)
+      throw LispErrNotList();
+
   iter = *pPtr;
 
+  const LispInt iNrMatchers = iMatchers.size();
   for (LispInt i=0;i<iNrMatchers;i++,++iter)
   {
     if (!iter.getObj())
-      return LispFalse;
+      return false;
     if (!iMatchers[i]->ArgumentMatches(aEnvironment,*iter,arguments))
-      return LispFalse;
+      return false;
   }
   if (iter.getObj())
-    return LispFalse;
-  return LispTrue;
+    return false;
+  return true;
 }
 
-LispInt YacasPatternPredicateBase::LookUp(LispString * aVariable)
+LispInt YacasPatternPredicateBase::LookUp(const LispString * aVariable)
 {
-    LispInt i;
-    for (i=0;i<iVariables.Size();i++)
-    {
+    const std::size_t n = iVariables.size();
+    for (std::size_t i = 0; i < n; ++i)
         if (iVariables[i] == aVariable)
-        {
             return i;
-        }
-    }
-    iVariables.Append(aVariable);
-    return iVariables.Size()-1;
+
+    aVariable->iReferenceCount += 1;
+    iVariables.push_back(aVariable);
+    return iVariables.size() - 1;
 }
 
 
-YacasParamMatcherBase* YacasPatternPredicateBase::MakeParamMatcher(LispEnvironment& aEnvironment, LispObject* aPattern)
+const YacasParamMatcherBase* YacasPatternPredicateBase::MakeParamMatcher(LispEnvironment& aEnvironment, LispObject* aPattern)
 {
     if (!aPattern)
-        return NULL;
+        return nullptr;
 
     if (aPattern->Number(aEnvironment.Precision()))
-    {
         return NEW MatchNumber(aPattern->Number(aEnvironment.Precision()));
-    }
 
     // Deal with atoms
     if (aPattern->String())
-    {
         return NEW MatchAtom(aPattern->String());
-    }
 
     // Else it must be a sublist
     if (aPattern->SubList())
     {
         // See if it is a variable template:
         LispPtr* sublist = aPattern->SubList();
-        LISPASSERT(sublist);
+        assert(sublist);
 
         LispInt num = InternalListLength(*sublist);
 
@@ -199,133 +147,133 @@ YacasParamMatcherBase* YacasPatternPredicateBase::MakeParamMatcher(LispEnvironme
                         while (!!last->Nixed())
                             last = last->Nixed();
 
-                        last->Nixed() = (LispAtom::New(aEnvironment,second->String()->c_str()));
+                        last->Nixed() = LispAtom::New(aEnvironment, *second->String());
 
                         LispPtr pred(LispSubList::New(third));
                         DBG_( third->Nixed()->SetFileAndLine(second->iFileName,second->iLine); )
                         DBG_( pred->SetFileAndLine(head->iFileName,head->iLine); )
-                        iPredicates.Append(pred);
+                        iPredicates.push_back(pred);
                     }
                     return NEW MatchVariable(index);
                 }
             }
         }
 
-        YacasParamMatcherBase** matchers = PlatAllocN<YacasParamMatcherBase*>(num);
+        std::vector<const YacasParamMatcherBase*> matchers;
+        matchers.reserve(num);
         LispIterator iter(*sublist);
-        for (LispInt i=0;i<num;i++,++iter)
-        {
-            matchers[i] = MakeParamMatcher(aEnvironment,iter.getObj());
-            LISPASSERT(matchers[i]);
+        for (LispInt i = 0; i < num; ++i, ++iter) {
+            matchers.push_back(MakeParamMatcher(aEnvironment,iter.getObj()));
+            assert(matchers[i]);
         }
-    return NEW MatchSubList(matchers, num);
+        return NEW MatchSubList(std::move(matchers));
     }
- 
-    return NULL;
+
+    return nullptr;
 }
 
-YacasPatternPredicateBase::YacasPatternPredicateBase(LispEnvironment& aEnvironment, LispPtr& aPattern,
-                                                     LispPtr& aPostPredicate)
-  : iParamMatchers(),iVariables(),iPredicates()
+YacasPatternPredicateBase::YacasPatternPredicateBase(
+    LispEnvironment& aEnvironment,
+    LispPtr& aPattern,
+    LispPtr& aPostPredicate)
 {
-    LispIterator iter(aPattern);
-    for ( ; iter.getObj(); ++iter)
-  {
-        YacasParamMatcherBase* matcher = MakeParamMatcher(aEnvironment,iter.getObj());
-        LISPASSERT(matcher!=NULL);
-        iParamMatchers.Append(matcher);
-  }
-  LispPtr post(aPostPredicate);
-  iPredicates.Append(post);
+    for (LispIterator iter(aPattern); iter.getObj(); ++iter) {
+        const YacasParamMatcherBase* matcher = MakeParamMatcher(aEnvironment, iter.getObj());
+        assert(matcher!=nullptr);
+        iParamMatchers.push_back(matcher);
+    }
+
+    iPredicates.push_back(aPostPredicate);
 }
 
-LispBoolean YacasPatternPredicateBase::Matches(LispEnvironment& aEnvironment,
+bool YacasPatternPredicateBase::Matches(LispEnvironment& aEnvironment,
                                               LispPtr& aArguments)
 {
-    LispPtr* arguments = NULL;
-    if (iVariables.Size() > 0)
-    arguments = NEW LispPtr[iVariables.Size()];
+    LispPtr* arguments = nullptr;
+    if (!iVariables.empty())
+        arguments = NEW LispPtr[iVariables.size()];
     LocalArgs args(arguments); //Deal with destruction
     LispIterator iter(aArguments);
-    for (LispInt i=0;i<iParamMatchers.Size();i++,++iter)
+    const std::size_t n = iParamMatchers.size();
+    for (std::size_t i = 0; i < n; ++i, ++iter)
   {
         if (!iter.getObj())
-            return LispFalse;
+            return false;
         if (!iParamMatchers[i]->ArgumentMatches(aEnvironment,*iter,arguments))
         {
-            return LispFalse;
+            return false;
         }
   }
     if (iter.getObj())
-        return LispFalse;
+        return false;
 
     {
         // set the local variables.
-        LispLocalFrame frame(aEnvironment,LispFalse);
+        LispLocalFrame frame(aEnvironment,false);
 
         SetPatternVariables(aEnvironment,arguments);
 
         // do the predicates
         if (!CheckPredicates(aEnvironment))
-            return LispFalse;
+            return false;
     }
 
     // set the local variables for sure now
     SetPatternVariables(aEnvironment,arguments);
- 
-    return LispTrue;
+
+    return true;
 }
 
 
 
 
-LispBoolean YacasPatternPredicateBase::Matches(LispEnvironment& aEnvironment,
+bool YacasPatternPredicateBase::Matches(LispEnvironment& aEnvironment,
                                               LispPtr* aArguments)
 {
-    LispInt i;
-
-    LispPtr* arguments = NULL;
-    if (iVariables.Size() > 0)
-        arguments = NEW LispPtr[iVariables.Size()];
+    LispPtr* arguments = nullptr;
+    if (!iVariables.empty())
+        arguments = NEW LispPtr[iVariables.size()];
     LocalArgs args(arguments); //Deal with destruction
 
-    for (i=0;i<iParamMatchers.Size();i++)
+    const std::size_t n = iParamMatchers.size();
+    for (std::size_t i = 0; i < n; ++i)
     {
         if (!iParamMatchers[i]->ArgumentMatches(aEnvironment,aArguments[i],arguments))
         {
-            return LispFalse;
+            return false;
         }
     }
 
     {
         // set the local variables.
-        LispLocalFrame frame(aEnvironment,LispFalse);
+        LispLocalFrame frame(aEnvironment,false);
         SetPatternVariables(aEnvironment,arguments);
 
         // do the predicates
         if (!CheckPredicates(aEnvironment))
-            return LispFalse;
+            return false;
     }
 
     // set the local variables for sure now
     SetPatternVariables(aEnvironment,arguments);
-    return LispTrue;
+    return true;
 }
 
 
 
-LispBoolean YacasPatternPredicateBase::CheckPredicates(LispEnvironment& aEnvironment)
+bool YacasPatternPredicateBase::CheckPredicates(LispEnvironment& aEnvironment)
 {
-  for (LispInt i=0;i<iPredicates.Size();i++)
+    const std::size_t n = iPredicates.size();
+  for (std::size_t i = 0; i < n; ++i)
   {
     LispPtr pred;
-    InternalEval(aEnvironment, pred, iPredicates[i]);
+    aEnvironment.iEvaluator->Eval(aEnvironment, pred, iPredicates[i]);
     if (IsFalse(aEnvironment, pred))
     {
-      return LispFalse;
+      return false;
     }
     // If the result is not False, it should be True, else probably something is wrong (the expression returned unevaluated)
-    LispBoolean isTrue = IsTrue(aEnvironment, pred);
+    bool isTrue = IsTrue(aEnvironment, pred);
     if (!isTrue)
     {
 #define LIM_AL 60
@@ -339,35 +287,36 @@ LispBoolean YacasPatternPredicateBase::CheckPredicates(LispEnvironment& aEnviron
 #endif
 
 
-      aEnvironment.iErrorOutput.Write("The predicate\n\t");
+      aEnvironment.iErrorOutput << "The predicate\n\t";
       PrintExpression(strout, iPredicates[i], aEnvironment, LIM_AL);
-      aEnvironment.iErrorOutput.Write(strout.c_str());
-      aEnvironment.iErrorOutput.Write("\nevaluated to\n\t");
+      aEnvironment.iErrorOutput << strout;
+      aEnvironment.iErrorOutput << "\nevaluated to\n\t";
       PrintExpression(strout, pred, aEnvironment, LIM_AL);
-      aEnvironment.iErrorOutput.Write(strout.c_str());
-      aEnvironment.iErrorOutput.Write("\n");
+      aEnvironment.iErrorOutput << strout << '\n';
 
-
-      CHK2(isTrue,KLispErrNonBooleanPredicateInPattern);
+      ShowStack(aEnvironment);
+      throw LispErrMaxRecurseDepthReached();
     }
   }
-  return LispTrue;
+  return true;
 }
- 
+
 
 void YacasPatternPredicateBase::SetPatternVariables(LispEnvironment& aEnvironment,
                                                     LispPtr* arguments)
 {
-    LispInt i;
-    for (i=0;i<iVariables.Size();i++)
-    {
-        // set the variable to the new value
+    const std::size_t n = iVariables.size();
+    for (std::size_t i = 0; i < n; ++i)
         aEnvironment.NewLocal(iVariables[i],arguments[i]);
-    }
 }
 
 
 YacasPatternPredicateBase::~YacasPatternPredicateBase()
 {
-}
+    for (const LispString* p: iVariables)
+        if (--p->iReferenceCount == 0)
+            delete p;
 
+    for (const YacasParamMatcherBase* p: iParamMatchers)
+        delete p;
+}
