@@ -1,19 +1,19 @@
+#include "yacas/yacasprivate.h"
+#include "yacas/unixcommandline.h"
 
-#include <time.h>
-#include <stdio.h>
 #include <sys/ioctl.h>
 
-#include "yacasprivate.h"
-#include "unixcommandline.h"
+#include <iostream>
+#include <fstream>
+
+
 
 void CUnixCommandLine::NewLine()
 {
     _cursor_line = 0;
     _last_line = 0;
 
-    printf("\n");
-
-    fflush(stdout);
+    std::cout << std::endl;
 }
 
 void CUnixCommandLine::Pause()
@@ -22,54 +22,53 @@ void CUnixCommandLine::Pause()
     while (clock()<i);
 }
 
-void CUnixCommandLine::ShowLine(
-    const LispChar* prompt,
-    LispInt promptlen,
-    LispInt cursor)
+void CUnixCommandLine::ShowLine(const std::string& prompt, unsigned cursor)
 {
     struct winsize w;
     ioctl(0, TIOCGWINSZ, &w);
 
-    const LispInt l = (cursor + promptlen) / w.ws_col;
-    const LispInt c = (cursor + promptlen) % w.ws_col;
+    const std::size_t prompt_len = prompt.length();
+
+    const LispInt l = (cursor + prompt_len) / w.ws_col;
+    const LispInt c = (cursor + prompt_len) % w.ws_col;
 
     if (_cursor_line)
-        printf("\x1b[%dF", _cursor_line);
+        std::cout << "\x1b[" << _cursor_line << "F";
 
-    if (iFullLineDirty) {
+    if (full_line_dirty) {
         if (_last_line)
-            printf("\x1b[%dB", _last_line);
+            std::cout << "\x1b[" << _last_line << "B";
 
         for (LispInt i = 0; i < _last_line; ++i)
-            printf("\r\x1b[K\x1b[F");
+            std::cout << "\r\x1b[K\x1b[F";
 
-        printf("\r\x1b[K\x1b[K%s%s", prompt, &iSubLine[0]);
+        std::cout << "\r\x1b[K\x1b[K" << prompt << iSubLine;
 
-        if ((promptlen + strlen(&iSubLine[0])) % w.ws_col == 0)
-            printf("\n");
+        if ((prompt_len + iSubLine.size()) % w.ws_col == 0)
+            std::cout << "\n";
 
-        _last_line = (promptlen + strlen(&iSubLine[0])) / w.ws_col;
+        _last_line = (prompt_len + iSubLine.size()) / w.ws_col;
         if (_last_line)
-            printf("\x1b[%dF", _last_line);
+            std::cout << "\x1b[" << _last_line << "F";
     }
 
     if (l)
-        printf("\r\x1b[%dB", l);
+        std::cout << "\r\x1b[" << l << "B";
 
-    printf("\r\x1b[%dG", c + 1);
+    std::cout << "\r\x1b[" << c + 1 << "G";
 
-    fflush(stdout);
+    std::cout << std::flush;
 
     _cursor_line = l;
 
-    iFullLineDirty = 0;
+    full_line_dirty = false;
 }
 
 
 CUnixCommandLine::CUnixCommandLine():
-    _cursor_line(0), 
-    _last_line(0), 
-    iMaxLines(1024)
+    _cursor_line(0),
+    _last_line(0),
+    _max_lines(1024)
 {
     /* set termio so we can do our own input processing */
     tcgetattr(0, &orig_termio);
@@ -91,70 +90,47 @@ CUnixCommandLine::CUnixCommandLine():
     rl_termio.c_cc[VSUSP] = 0;
     tcsetattr(0, TCSADRAIN, &rl_termio);
 
-    char fname[256];
-#ifdef HAVE_VSNPRINTF
-    snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
-#else
-    sprintf(fname,"%s/.yacas_history",getenv("HOME"));
-#endif
-    FILE*f=fopen(fname,"r");
-    if (f)
-    {
-        if(f)
-        {
-#define BufSz 256
-            char buff[BufSz];
-            while(fgets(buff,BufSz-2,f))
-            {
-                int i;
-                for(i=0;buff[i] && buff[i] != '\n';++i)
-                    ;
-                buff[i++] = '\0';
-                LispString * ptr = NEW LispString(buff);
-                iHistoryList.Append(ptr);
- 
-            }
-            fclose(f);
-        }
-    }
-    iMaxLines = 1024;
+    const char* home_dir = getenv("HOME");
+
+    if (!home_dir)
+        return;
+
+    const std::string fname = std::string(home_dir) + "/.yacas_history";
+
+    std::ifstream is(fname.c_str());
+
+    std::string line;
+    while (std::getline(is, line))
+        iHistoryList.Append(line);
 }
 
 CUnixCommandLine::~CUnixCommandLine()
 {
     tcsetattr(0, TCSADRAIN, &orig_termio);
-    char fname[256];
-#ifdef HAVE_VSNPRINTF
-    snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
-#else
-    sprintf(fname,"%s/.yacas_history",getenv("HOME"));
-#endif
- 
-    FILE*f=fopen(fname,"w");
-    if (f)
-    {
-        int i;
-        int from=0;
-        if (iMaxLines>=0)
-        {
-            if (iHistoryList.NrLines()>iMaxLines)
-            {
-                from = iHistoryList.NrLines()-iMaxLines;
-            }
-        }
-        for (i=from;i<iHistoryList.NrLines();i++)
-        {
-          LispString * ptr = iHistoryList.GetLine(i);
 
-            fprintf(f,"%s\n",ptr->c_str());
-        }
-        fclose(f);
+    const char* home_dir = getenv("HOME");
+
+    if (!home_dir)
+        return;
+
+    const std::string fname = std::string(home_dir) + "/.yacas_history";
+
+    std::ofstream os(fname.c_str());
+
+    if (os) {
+        std::size_t from = 0;
+
+        if (_max_lines > 0 && iHistoryList.NrLines() > _max_lines)
+            from = iHistoryList.NrLines() - _max_lines;
+
+        for (std::size_t i = from; i < iHistoryList.NrLines(); ++i)
+            os << iHistoryList.GetLine(i) << "\n";
     }
 }
 
-void CUnixCommandLine::MaxHistoryLinesSaved(LispInt aNrLines)
+void CUnixCommandLine::MaxHistoryLinesSaved(std::size_t n)
 {
-    iMaxLines = aNrLines;
+    _max_lines = n;
 }
 
 
@@ -196,6 +172,9 @@ LispInt CUnixCommandLine::GetKey()
         case  8:              /* ^H  (unix backspace) */
             c = eBackSpace;
             break;
+        case 11:              /* ^K (unix kill to the end of line */
+            c = eKill;
+            break;
         case 033:
             {
                 c = getc(stdin); /* check for CSI */
@@ -206,6 +185,7 @@ LispInt CUnixCommandLine::GetKey()
                     c=eEscape;
                     break;
                 case '[':
+                case 79:
                     {
                         c = getc(stdin); /* get command character */
                         switch (c)
@@ -238,6 +218,3 @@ LispInt CUnixCommandLine::GetKey()
     }
     return c;
 }
-
-
-
