@@ -1,33 +1,30 @@
-
-
 /* Implementation of the number classes (the functionality used
  * by yacas any way
  */
 
-#include "yacasprivate.h"
-#include "yacasbase.h"
-#include "numbers.h"
-#include "standard.h"
-#include "anumber.h"
-#include "platmath.h"
-#include "lisperror.h"
-#include "errors.h"
+#include "yacas/yacasprivate.h"
+#include "yacas/yacasbase.h"
+#include "yacas/numbers.h"
+#include "yacas/standard.h"
+#include "yacas/anumber.h"
+#include "yacas/platmath.h"
+#include "yacas/lisperror.h"
+#include "yacas/errors.h"
 
 #ifdef HAVE_CONFIG_H
   #include <config.h>
 #endif
 
-#ifdef HAVE_STDIO_H
-  #include <stdio.h> // Safe, only included if HAVE_STDIO_H defined
-#endif
-
-#define BITS_TO_DIGITS(x,n) (bits_to_digits(x,n))
+#include <iomanip>
+#include <iostream>
+#include <sstream>
+#include <algorithm>
 
 static LispObject* FloatToString(ANumber& aInt, LispEnvironment& aEnvironment, LispInt aBase = 10);
 
 LispInt NumericSupportForMantissa()
 {
-  return LispTrue;
+  return true;
 }
 
 
@@ -45,8 +42,10 @@ LispObject* GcdInteger(LispObject* int1, LispObject* int2,
 {
   BigNumber* i1 = int1->Number(0);
   BigNumber* i2 = int2->Number(0);
-  Check(i1->iNumber->iExp == 0, KLispErrNotInteger);
-  Check(i2->iNumber->iExp == 0, KLispErrNotInteger);
+
+  if (i1->iNumber->iExp != 0 || i2->iNumber->iExp != 0)
+      throw LispErrNotInteger();
+
   BigNumber* res = NEW BigNumber();
   BaseGcd(*res->iNumber,*i1->iNumber,*i2->iNumber);
   return NEW LispNumber(res);
@@ -69,28 +68,15 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
     ANumber dummy(10);
 
     LispInt requiredDigits = WordDigits(sum.iPrecision, 10)+
-        x2.Size()-x2.iExp+1;
+        x2.size()-x2.iExp+1;
 //    printf("WordDigits=%d\n",requiredDigits);
-//    printf("[%d,%d]:",x.Size()-x.iExp,x.iExp);
+//    printf("[%d,%d]:",x.size()-x.iExp,x.iExp);
 
     // While (term>epsilon)
     while (1 /* Significant(term)*/)
     {
- 
-#ifdef CORRECT_DIVISION
+        if (!Significant(term)) break;
 
-/*
-        LispInt significantDigits = WordDigits(term.iPrecision, 10);
-        NormalizeFloat(term,significantDigits);
-        if ((-term.iTensExp) > term.iPrecision+2)
-        {
-          break;
-        }
-*/
-        if (!Significant(term)) break;
-#else
-        if (!Significant(term)) break;
-#endif
         ANumber orig(sum.iPrecision);
 
         //   term <- term*x^2/((i+1)(i+2))
@@ -101,7 +87,7 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
             LispInt toDunk = term.iExp - requiredDigits;
             if (toDunk > 0)
             {
-                term.Delete(0,toDunk);
+                term.erase(term.begin(),term.begin()+toDunk);
                 term.iExp = requiredDigits;
             }
         }
@@ -119,7 +105,7 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
         Divide(term, dummy, orig, i);
 
         //   negate term
-        Negate(term);
+        term.Negate();
         //   sum <- sum+term
         orig.CopyFrom(sum);
         Add(sum, orig, term);
@@ -127,7 +113,7 @@ static void Trigonometry(ANumber& x,ANumber& i,ANumber& sum,ANumber& term)
 
     }
 
-//    printf("[%d,%d]:",sum.Size()-sum.iExp,sum.iExp);
+//    printf("[%d,%d]:",sum.size()-sum.iExp,sum.iExp);
 }
 
 static void SinFloat(ANumber& aResult, ANumber& x)
@@ -222,7 +208,7 @@ LispObject* ArcSinFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt
   // how else do I get an ANumber from the result of PlatArcSin()?
   ANumber x(aPrecision);  // dummy variable
 //  ANumber q("10", aPrecision);  // initial value must be "significant"
- 
+
   ANumber q( aPrecision);  // initial value must be "significant"
   {
     ANumber x(aPrecision);
@@ -231,18 +217,18 @@ LispObject* ArcSinFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt
     SinFloat(s,x);
     ANumber orig(aPrecision);
     orig.CopyFrom(*int1->Number(aPrecision)->iNumber);
-    Negate(orig);
+    orig.Negate();
     Add(q,s,orig);
   }
 
   ANumber s(aPrecision);
   ANumber c(aPrecision);
- 
+
   while (Significant(q))
   {
     x.CopyFrom(result);
     SinFloat(s, x);
-    Negate(s);
+    s.Negate();
     x.CopyFrom(s);
     ANumber y(*int1->Number(aPrecision)->iNumber);
 //PrintNumber("y = ",y);
@@ -275,7 +261,7 @@ static void ExpFloat(ANumber& aResult, ANumber& x)
     ANumber dummy(10);
 
     LispInt requiredDigits = WordDigits(aResult.iPrecision, 10)+
-        x.Size()-x.iExp+1;
+        x.size()-x.iExp+1;
 
     // While (term>epsilon)
     while (Significant(term))
@@ -286,12 +272,12 @@ static void ExpFloat(ANumber& aResult, ANumber& x)
             LispInt toDunk = term.iExp - requiredDigits;
             if (toDunk > 0)
             {
-                term.Delete(0,toDunk);
+                term.erase(term.begin(),term.begin()+toDunk);
                 term.iExp = requiredDigits;
             }
         }
 
- 
+
         //   i <- i+1
         BaseAdd(i, one, WordBase);
 
@@ -317,56 +303,53 @@ LispObject* ExpFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt aPr
 
 LispObject* PowerFloat(LispObject* int1, LispObject* int2, LispEnvironment& aEnvironment,LispInt aPrecision)
 {
-    // If is integer
-    if (int2->Number(aPrecision)->iNumber->iExp == 0)
+    if (int2->Number(aPrecision)->iNumber->iExp != 0)
+        throw LispErrNotInteger();
+
+    // Raising to the power of an integer can be done fastest by squaring
+    // and bitshifting: x^(a+b) = x^a*x^b . Then, regarding each bit
+    // in y (seen as a binary number) as added, the algorithm becomes:
+    //
+    ANumber x(*int1->Number(aPrecision)->iNumber);
+    ANumber y(*int2->Number(aPrecision)->iNumber);
+    bool neg = y.iNegative;
+    y.iNegative=false;
+
+    // result <- 1
+    ANumber result("1",aPrecision);
+    // base <- x
+    ANumber base(aPrecision);
+    base.CopyFrom(x);
+
+    ANumber copy(aPrecision);
+
+    // while (y!=0)
+    while (!y.IsZero())
     {
-        // Raising to the power of an integer can be done fastest by squaring
-        // and bitshifting: x^(a+b) = x^a*x^b . Then, regarding each bit
-        // in y (seen as a binary number) as added, the algorithm becomes:
-        //
-        ANumber x(*int1->Number(aPrecision)->iNumber);
-        ANumber y(*int2->Number(aPrecision)->iNumber);
-        LispBoolean neg = y.iNegative;
-        y.iNegative=LispFalse;
- 
-        // result <- 1
-        ANumber result("1",aPrecision);
-        // base <- x
-        ANumber base(aPrecision);
-        base.CopyFrom(x);
-
-        ANumber copy(aPrecision);
-
-        // while (y!=0)
-        while (!IsZero(y))
+        // if (y&1 != 0)
+        if ( (y[0] & 1) != 0)
         {
-            // if (y&1 != 0)
-            if ( (y[0] & 1) != 0)
-            {
-                // result <- result*base
-                copy.CopyFrom(result);
-                Multiply(result,copy,base);
-            }
-            // base <- base*base
-            copy.CopyFrom(base);
-            Multiply(base,copy,copy);
-            // y <- y>>1
-            BaseShiftRight(y,1);
-        }
-
-        if (neg)
-        {
-            ANumber one("1",aPrecision);
-            ANumber dummy(10);
+            // result <- result*base
             copy.CopyFrom(result);
-            Divide(result,dummy,one,copy);
+            Multiply(result,copy,base);
         }
- 
-        // result
-        return FloatToString(result, aEnvironment);
+        // base <- base*base
+        copy.CopyFrom(base);
+        Multiply(base,copy,copy);
+        // y <- y>>1
+        BaseShiftRight(y,1);
     }
-    Check(LispFalse, KLispErrNotInteger);
-  return 0;
+
+    if (neg)
+    {
+        ANumber one("1",aPrecision);
+        ANumber dummy(10);
+        copy.CopyFrom(result);
+        Divide(result,dummy,one,copy);
+    }
+
+    // result
+    return FloatToString(result, aEnvironment);
 }
 
 
@@ -388,7 +371,7 @@ LispObject* SqrtFloat(LispObject* int1, LispEnvironment& aEnvironment,LispInt aP
 LispObject* ShiftLeft( LispObject* int1, LispObject* int2, LispEnvironment& aEnvironment,LispInt aPrecision)
 {
   BigNumber *number = NEW BigNumber();
-  LispInt bits = InternalAsciiToInt(int2->String());
+  LispInt bits = InternalAsciiToInt(*int2->String());
   number->ShiftLeft(*int1->Number(aPrecision),bits);
   return NEW LispNumber(number);
 }
@@ -397,21 +380,24 @@ LispObject* ShiftLeft( LispObject* int1, LispObject* int2, LispEnvironment& aEnv
 LispObject* ShiftRight( LispObject* int1, LispObject* int2, LispEnvironment& aEnvironment,LispInt aPrecision)
 {
   BigNumber *number = NEW BigNumber();
-  LispInt bits = InternalAsciiToInt(int2->String());
+  LispInt bits = InternalAsciiToInt(*int2->String());
   number->ShiftRight(*int1->Number(aPrecision),bits);
   return NEW LispNumber(number);
 }
 
-static void DivideInteger( ANumber& aQuotient, ANumber& aRemainder,
-                        LispChar * int1, LispChar * int2,
-                        LispInt aPrecision)
+static void DivideInteger(ANumber& aQuotient, ANumber& aRemainder,
+                          const LispChar* int1, const LispChar* int2,
+                          LispInt aPrecision)
 {
     ANumber a1(int1,aPrecision);
     ANumber a2(int2,aPrecision);
- 
-    Check(a1.iExp == 0, KLispErrNotInteger);
-    Check(a2.iExp == 0, KLispErrNotInteger);
-    Check(!IsZero(a2),KLispErrInvalidArg);
+
+    if (a1.iExp != 0 || a2.iExp != 0)
+          throw LispErrNotInteger();
+
+    if (a2.IsZero())
+          throw LispErrInvalidArg();
+
     IntegerDivide(aQuotient, aRemainder, a1, a2);
 }
 
@@ -431,14 +417,17 @@ static LispObject* FloatToString(ANumber& aInt,
 {
     LispString result;
     ANumberToString(result, aInt, aBase);
-    return LispAtom::New(aEnvironment, result.c_str());
+    return LispAtom::New(aEnvironment, result);
 }
 
 
 LispObject* LispFactorial(LispObject* int1, LispEnvironment& aEnvironment,LispInt aPrecision)
 {
-    LispInt nr = InternalAsciiToInt(int1->String());
-    Check(nr>=0,KLispErrInvalidArg);
+    LispInt nr = InternalAsciiToInt(*int1->String());
+
+    if (nr < 0)
+        throw LispErrInvalidArg();
+
     ANumber fac("1",aPrecision);
     LispInt i;
     for (i=2;i<=nr;i++)
@@ -469,7 +458,10 @@ void tree_factorial(ANumber& result, LispInt iLeft, LispInt iRight, LispInt aPre
 LispString * LispFactorial(LispChar * int1, LispHashTable& aHashTable,LispInt aPrecision)
 {
     LispInt nr = InternalAsciiToInt(int1);
-    Check(nr>=0,KLispErrInvalidArg);
+
+    if (nr < 0)
+        throw LispErrInvalidArg();
+
   ANumber fac("1",aPrecision);
   tree_factorial(fac, 1, nr, aPrecision);
     return FloatToString(fac, aEnvironment);
@@ -482,22 +474,22 @@ LispString * LispFactorial(LispChar * int1, LispHashTable& aHashTable,LispInt aP
 // this will use the new BigNumber/BigInt/BigFloat scheme
 
 BigNumber::BigNumber(const LispChar * aString,LispInt aBasePrecision,LispInt aBase)
- : iReferenceCount(),iPrecision(0),iType(KInt),iNumber(NULL)
+ : iReferenceCount(),iPrecision(0),iType(KInt),iNumber(nullptr)
 {
-  iNumber = NULL;
+  iNumber = nullptr;
   SetTo(aString, aBasePrecision, aBase);
 }
 BigNumber::BigNumber(const BigNumber& aOther)
- : iReferenceCount(),iPrecision(aOther.GetPrecision()),iType(KInt),iNumber(NULL)
+ : iReferenceCount(),iPrecision(aOther.GetPrecision()),iType(KInt),iNumber(nullptr)
 {
   iNumber = NEW ANumber(*aOther.iNumber);
   SetIsInteger(aOther.IsInt());
 }
 BigNumber::BigNumber(LispInt aPrecision)
- : iReferenceCount(),iPrecision(aPrecision),iType(KInt),iNumber(NULL)
+ : iReferenceCount(),iPrecision(aPrecision),iType(KInt),iNumber(nullptr)
 {
-  iNumber = NEW ANumber(BITS_TO_DIGITS(aPrecision,10));
-  SetIsInteger(LispTrue);
+  iNumber = NEW ANumber(bits_to_digits(aPrecision,10));
+  SetIsInteger(true);
 }
 
 BigNumber::~BigNumber()
@@ -525,7 +517,7 @@ void BigNumber::ToString(LispString& aResult, LispInt aBasePrecision, LispInt aB
 {
   ANumber num(*iNumber);
 //  num.CopyFrom(*iNumber);
- 
+
   //TODO this round-off code is not correct yet, but will work in most cases
   // This is a bit of a messy way to round off numbers. It is probably incorrect,
   // even. When precision drops one digit, it rounds off the last ten digits.
@@ -542,16 +534,17 @@ void BigNumber::ToString(LispString& aResult, LispInt aBasePrecision, LispInt aB
   {
     for(;;)
     {
-      LispInt i;
-      LispBoolean greaterOne = LispFalse;
-      if (num.iExp >= num.Size()) break;
-      for (i=num.iExp;i<num.Size();i++)
+
+      const LispInt ns = num.size();
+      bool greaterOne = false;
+      if (num.iExp >= LispInt(ns)) break;
+      for (LispInt i=num.iExp;i<ns;i++)
       {
         if (num[i] != 0)
         {
           if (!(i==num.iExp && num[i]<10000 && num.iTensExp == 0))
           {
-            greaterOne=LispTrue;
+            greaterOne=true;
             break;
           }
         }
@@ -567,19 +560,13 @@ void BigNumber::ToString(LispString& aResult, LispInt aBasePrecision, LispInt aB
 }
 double BigNumber::Double() const
 {
-// There are platforms that don't have strtod
-#ifdef HAVE_STRTOD
-  LispString str;
-  ANumber num(*iNumber);
-//  num.CopyFrom(*iNumber);
-  ANumberToString(str, num, 10);
-  char* endptr;
-  return strtod(str.c_str(),&endptr);
-#else
-  //FIXME
-  LISPASSERT(0);
-  return 0.0;
-#endif
+    LispString str;
+    ANumber num(*iNumber);
+    ANumberToString(str, num, 10);
+    std::istringstream is(str.c_str());
+    double d;
+    is >> d;
+    return d;
 }
 
 
@@ -600,7 +587,7 @@ void BigNumber::Multiply(const BigNumber& aX, const BigNumber& aY, LispInt aPrec
   if (aPrecision<aX.GetPrecision()) aPrecision=aX.GetPrecision();
   if (aPrecision<aY.GetPrecision()) aPrecision=aY.GetPrecision();
 
-  iNumber->ChangePrecision(BITS_TO_DIGITS(aPrecision,10));
+  iNumber->ChangePrecision(bits_to_digits(aPrecision,10));
 
 
 //  if (iNumber == aX.iNumber || iNumber == aY.iNumber)
@@ -651,7 +638,7 @@ void BigNumber::Negate(const BigNumber& aX)
   {
     iNumber->CopyFrom(*aX.iNumber);
   }
-  ::Negate(*iNumber);
+  iNumber->Negate();
   SetIsInteger(aX.IsInt());
 }
 
@@ -662,7 +649,7 @@ LispInt DividePrecision(const BigNumber& aX, const BigNumber& aY, LispInt aPreci
   LispInt p=1;
   if (aY.Sign()==0)
   {  // zero division, report and do nothing
-    RaiseError("BigNumber::Divide: zero division request ignored\n");
+    throw LispErrGeneric("BigNumber::Divide: zero division request ignored");
     p=0;
   }
   if (aX.IsInt())
@@ -715,7 +702,7 @@ void BigNumber::Divide(const BigNumber& aX, const BigNumber& aY, LispInt aPrecis
 
 /*
   iPrecision = DividePrecision(aX, aY, aPrecision);
-  LispInt digitPrecision = BITS_TO_DIGITS(iPrecision,10);
+  LispInt digitPrecision = bits_to_digits(iPrecision,10);
   iNumber->iPrecision = digitPrecision;
 */
 
@@ -723,7 +710,7 @@ void BigNumber::Divide(const BigNumber& aX, const BigNumber& aY, LispInt aPrecis
   if (aPrecision<aX.GetPrecision()) aPrecision=aX.GetPrecision();
   if (aPrecision<aY.GetPrecision()) aPrecision=aY.GetPrecision();
 
-  LispInt digitPrecision = BITS_TO_DIGITS(aPrecision,10);
+  LispInt digitPrecision = bits_to_digits(aPrecision,10);
   iPrecision = aPrecision;
   iNumber->iPrecision = digitPrecision;
 /* */
@@ -734,17 +721,20 @@ void BigNumber::Divide(const BigNumber& aX, const BigNumber& aY, LispInt aPrecis
 //  a2.CopyFrom(*aY.iNumber);
   ANumber remainder(digitPrecision);
 
-  Check(!IsZero(a2),KLispErrInvalidArg);
+  if (a2.IsZero())
+      throw LispErrInvalidArg();
+
   if (aX.IsInt() && aY.IsInt())
   {
-    Check(a1.iExp == 0, KLispErrNotInteger);
-    Check(a2.iExp == 0, KLispErrNotInteger);
-    SetIsInteger(LispTrue);
+      if (a1.iExp != 0 || a2.iExp != 0)
+          throw LispErrNotInteger();
+
+    SetIsInteger(true);
     ::IntegerDivide(*iNumber, remainder, a1, a2);
   }
   else
   {
-    SetIsInteger(LispFalse);
+    SetIsInteger(false);
     ::Divide(*iNumber,remainder,a1,a2);
   }
 }
@@ -766,7 +756,7 @@ void BigNumber::ShiftRight(const BigNumber& aX, LispInt aNrToShift)
 }
 void BigNumber::BitAnd(const BigNumber& aX, const BigNumber& aY)
 {
-  LispInt lenX=aX.iNumber->Size(), lenY=aY.iNumber->Size();
+  LispInt lenX=aX.iNumber->size(), lenY=aY.iNumber->size();
   LispInt min=lenX,max=lenY;
   if (min>max)
   {
@@ -774,13 +764,13 @@ void BigNumber::BitAnd(const BigNumber& aX, const BigNumber& aY)
     min=max;
     max=swap;
   }
-  iNumber->ResizeTo(min);
+  iNumber->resize(min);
   LispInt i;
   for (i=0;i<min;i++)  (*iNumber)[i] = (*aX.iNumber)[i] & (*aY.iNumber)[i];
 }
 void BigNumber::BitOr(const BigNumber& aX, const BigNumber& aY)
 {
-  LispInt lenX=(*aX.iNumber).Size(), lenY=(*aY.iNumber).Size();
+  LispInt lenX=(*aX.iNumber).size(), lenY=(*aY.iNumber).size();
   LispInt min=lenX,max=lenY;
   if (min>max)
   {
@@ -789,7 +779,7 @@ void BigNumber::BitOr(const BigNumber& aX, const BigNumber& aY)
     max=swap;
   }
 
-  iNumber->ResizeTo(max);
+  iNumber->resize(max);
 
   LispInt i;
   for (i=0;i<min;i++)    (*iNumber)[i] = (*aX.iNumber)[i] | (*aY.iNumber)[i];
@@ -798,7 +788,7 @@ void BigNumber::BitOr(const BigNumber& aX, const BigNumber& aY)
 }
 void BigNumber::BitXor(const BigNumber& aX, const BigNumber& aY)
 {
-  LispInt lenX=(*aX.iNumber).Size(), lenY=(*aY.iNumber).Size();
+  LispInt lenX=(*aX.iNumber).size(), lenY=(*aY.iNumber).size();
   LispInt min=lenX,max=lenY;
   if (min>max)
   {
@@ -806,8 +796,8 @@ void BigNumber::BitXor(const BigNumber& aX, const BigNumber& aY)
     min=max;
     max=swap;
   }
- 
-  iNumber->ResizeTo(max);
+
+  iNumber->resize(max);
 
   LispInt i;
   for (i=0;i<min;i++)  (*iNumber)[i] = (*aX.iNumber)[i] ^ (*aY.iNumber)[i];
@@ -817,9 +807,9 @@ void BigNumber::BitXor(const BigNumber& aX, const BigNumber& aY)
 
 void BigNumber::BitNot(const BigNumber& aX)
 {// FIXME?
-  LispInt len=(*aX.iNumber).Size();
- 
-  iNumber->ResizeTo(len);
+  LispInt len=(*aX.iNumber).size();
+
+  iNumber->resize(len);
 
   LispInt i;
   for (i=0;i<len;i++)  (*iNumber)[i] = ~((*aX.iNumber)[i]);
@@ -834,7 +824,7 @@ signed long BigNumber::BitCount() const
   int low=0;
   int high = 0;
   int i;
-  for (i=0;i<iNumber->Size();i++)
+  for (i=0;i<iNumber->size();i++)
   {
     if ((*iNumber)[i] != 0)
     {
@@ -847,7 +837,7 @@ signed long BigNumber::BitCount() const
   return bits;
 */
 
-  if (IsZero(*iNumber)) return 0;//-(1L<<30);
+  if (iNumber->IsZero()) return 0;//-(1L<<30);
   ANumber num(*iNumber);
 //  num.CopyFrom(*iNumber);
 
@@ -857,7 +847,7 @@ signed long BigNumber::BitCount() const
     PlatWord zero=0;
     while(num.iExp<digs)
     {
-        num.Insert(0,zero);
+        num.insert(num.begin(),zero);
         num.iExp++;
     }
   }
@@ -873,7 +863,7 @@ signed long BigNumber::BitCount() const
     num.iTensExp--;
   }
 
-  LispInt i,nr=num.Size();
+  LispInt i,nr=num.size();
   for (i=nr-1;i>=0;i--)
   {
     if (num[i] != 0) break;
@@ -893,24 +883,17 @@ signed long BigNumber::BitCount() const
 LispInt BigNumber::Sign() const
 {
   if (iNumber->iNegative) return -1;
-  if (IsZero(*iNumber)) return 0;
+  if (iNumber->IsZero()) return 0;
   return 1;
 }
 
 
-void BigNumber::DumpDebugInfo()
+void BigNumber::DumpDebugInfo() const
 {
-#ifdef HAVE_STDIO_H
-  if (!iNumber)
-  {
-    printf("No number representation\n");
-  }
-  else
-  {
-    PrintNumber("Number:",*iNumber);
-  }
-#else
-#endif
+    if (!iNumber)
+        std::cout << "No number representation\n";
+    else
+        iNumber->Print("Number:");
 }
 
 
@@ -921,9 +904,12 @@ void BigNumber::Mod(const BigNumber& aY, const BigNumber& aZ)
     ANumber a2(*aZ.iNumber);
 //    a1.CopyFrom(*aY.iNumber);
 //    a2.CopyFrom(*aZ.iNumber);
-    Check(a1.iExp == 0, KLispErrNotInteger);
-    Check(a2.iExp == 0, KLispErrNotInteger);
-    Check(!IsZero(a2),KLispErrInvalidArg);
+
+    if (a1.iExp != 0 || a2.iExp != 0)
+          throw LispErrNotInteger();
+
+    if (a2.IsZero())
+          throw LispErrInvalidArg();
 
     ANumber quotient(static_cast<LispInt>(0));
     ::IntegerDivide(quotient, *iNumber, a1, a2);
@@ -934,7 +920,7 @@ void BigNumber::Mod(const BigNumber& aY, const BigNumber& aZ)
 //      a3.CopyFrom(*iNumber);
       ::Add(*iNumber, a3, a2);
     }
-    SetIsInteger(LispTrue);
+    SetIsInteger(true);
 }
 
 void BigNumber::Floor(const BigNumber& aX)
@@ -971,15 +957,15 @@ void BigNumber::Floor(const BigNumber& aX)
     }
     iNumber->ChangePrecision(iNumber->iPrecision);
     LispInt i=0;
-    LispInt fraciszero=LispTrue;
+    LispInt fraciszero=true;
     while (i<iNumber->iExp && fraciszero)
     {
         PlatWord digit = (*iNumber)[i];
         if (digit != 0)
-            fraciszero=LispFalse;
+            fraciszero=false;
         i++;
     }
-    iNumber->Delete(0,iNumber->iExp);
+    iNumber->erase(iNumber->begin(),iNumber->begin()+iNumber->iExp);
     iNumber->iExp=0;
 
     if (iNumber->iNegative && !fraciszero)
@@ -989,7 +975,7 @@ void BigNumber::Floor(const BigNumber& aX)
         ANumber minone("-1",10);
         ::Add(*iNumber,orig,minone);
     }
-    SetIsInteger(LispTrue);
+    SetIsInteger(true);
 }
 
 
@@ -1001,7 +987,7 @@ void BigNumber::Precision(LispInt aPrecision)
   }
   else
   {
-    iNumber->ChangePrecision(BITS_TO_DIGITS(aPrecision,10));
+    iNumber->ChangePrecision(bits_to_digits(aPrecision,10));
   }
   SetIsInteger(iNumber->iExp == 0 && iNumber->iTensExp == 0);
   iPrecision = aPrecision;
@@ -1009,24 +995,24 @@ void BigNumber::Precision(LispInt aPrecision)
 
 
 //basic object manipulation
-LispBoolean BigNumber::Equals(const BigNumber& aOther) const
+bool BigNumber::Equals(const BigNumber& aOther) const
 {
 
   if (iNumber->iExp == aOther.iNumber->iExp)
   {
     iNumber->DropTrailZeroes();
     aOther.iNumber->DropTrailZeroes();
- 
-    if (IsZero(*iNumber))
-        iNumber->iNegative = LispFalse;
-    if (IsZero(*aOther.iNumber))
-        aOther.iNumber->iNegative = LispFalse;
+
+    if (iNumber->IsZero())
+        iNumber->iNegative = false;
+    if (aOther.iNumber->IsZero())
+        aOther.iNumber->iNegative = false;
     if (iNumber->ExactlyEqual(*aOther.iNumber))
-      return LispTrue;
+      return true;
     if (IsInt())
-      return LispFalse;
+      return false;
     if (aOther.iNumber->iNegative != iNumber->iNegative)
-      return LispFalse;
+      return false;
     }
 
   {
@@ -1042,9 +1028,8 @@ LispBoolean BigNumber::Equals(const BigNumber& aOther) const
     BigNumber diff;
     BigNumber otherNeg;
     otherNeg.Negate(aOther);
-    diff.Add(*this,otherNeg,BITS_TO_DIGITS(precision,10));
+    diff.Add(*this,otherNeg,bits_to_digits(precision,10));
 
-#ifdef CORRECT_DIVISION
     // if the numbers are float, make sure they are normalized
     if (diff.iNumber->iExp || diff.iNumber->iTensExp)
     {
@@ -1055,26 +1040,24 @@ LispBoolean BigNumber::Equals(const BigNumber& aOther) const
         pr = aOther.iPrecision;
       NormalizeFloat(*diff.iNumber,WordDigits(pr, 10));
     }
-#endif // CORRECT_DIVISION
-
 
     return !Significant(*diff.iNumber);
   }
 }
 
 
-LispBoolean BigNumber::IsInt() const
+bool BigNumber::IsInt() const
 {
   return (iType == KInt);
 }
 
 
-LispBoolean BigNumber::IsIntValue() const
+bool BigNumber::IsIntValue() const
 {
 //FIXME I need to round first to get more reliable results.
-  if (IsInt()) return LispTrue;
+  if (IsInt()) return true;
   iNumber->DropTrailZeroes();
-  if (iNumber->iExp == 0 && iNumber->iTensExp == 0) return LispTrue;
+  if (iNumber->iExp == 0 && iNumber->iTensExp == 0) return true;
   BigNumber num(iPrecision);
   num.Floor(*this);
   return Equals(num);
@@ -1082,12 +1065,12 @@ LispBoolean BigNumber::IsIntValue() const
 }
 
 
-LispBoolean BigNumber::IsSmall() const
+bool BigNumber::IsSmall() const
 {
   if (IsInt())
   {
-    PlatWord* ptr = &((*iNumber)[iNumber->Size()-1]);
-    LispInt nr=iNumber->Size();
+    PlatWord* ptr = &((*iNumber)[iNumber->size()-1]);
+    LispInt nr=iNumber->size();
     while (nr>1 && *ptr == 0) {ptr--;nr--;}
     return (nr <= iNumber->iExp+1);
   }
@@ -1119,9 +1102,9 @@ void BigNumber::BecomeInt()
     BaseDivideInt(*iNumber,10, WordBase, carry);
     iNumber->iTensExp++;
   }
- 
+
   iNumber->ChangePrecision(0);
-  SetIsInteger(LispTrue);
+  SetIsInteger(true);
 }
 
 /// Transform integer to float, setting a given bit precision.
@@ -1133,13 +1116,13 @@ void BigNumber::BecomeFloat(LispInt aPrecision)
     LispInt precision = aPrecision;
     if (iPrecision > aPrecision)
       precision = iPrecision;
-    iNumber->ChangePrecision(BITS_TO_DIGITS(precision,10));  // is this OK or ChangePrecision means floating-point precision?
-    SetIsInteger(LispFalse);
+    iNumber->ChangePrecision(bits_to_digits(precision,10));  // is this OK or ChangePrecision means floating-point precision?
+    SetIsInteger(false);
   }
 }
 
 
-LispBoolean BigNumber::LessThan(const BigNumber& aOther) const
+bool BigNumber::LessThan(const BigNumber& aOther) const
 {
   ANumber a1(*this->iNumber);
 //  a1.CopyFrom(*this->iNumber);
@@ -1154,48 +1137,22 @@ LispBoolean BigNumber::LessThan(const BigNumber& aOther) const
 // assign from a platform type
 void BigNumber::SetTo(long aValue)
 {
-#ifdef HAVE_STDIO_H
-  char dummy[150];
-  //FIXME platform code
-#ifdef HAVE_VSNPRINTF
-  snprintf(dummy,150,"%ld",aValue);
-#else
-  sprintf(dummy,"%ld",aValue);
-#endif
-  SetTo(dummy,iPrecision,10);
-  SetIsInteger(LispTrue);
-#else
-  //FIXME
-  LISPASSERT(0);
-#endif
+    std::ostringstream buf;
+    buf << aValue;
+    SetTo(buf.str().c_str(), iPrecision, BASE10);
+    SetIsInteger(true);
 }
 
 
 void BigNumber::SetTo(double aValue)
 {
-#ifdef HAVE_STDIO_H
-  iPrecision = 53;  // standard double has 53 bits
-  char dummy[150];
-  //FIXME platform code
-  char format[20];
-#ifdef HAVE_VSNPRINTF
-  snprintf(format,20,"%%.%dg",iPrecision);
-  snprintf(dummy,150,format,aValue);
-#else
-  sprintf(format,"%%.%dg",iPrecision);
-  sprintf(dummy,format,aValue);
-#endif
-  SetTo(dummy,iPrecision,BASE10);
-  SetIsInteger(LispFalse);
-//  if (iNumber->iExp > 1)
-//    iNumber->RoundBits();
-#else
-  //FIXME
-  LISPASSERT(0);
-#endif
+    std::ostringstream buf;
+    buf << std::setprecision(53) << aValue;
+    SetTo(buf.str().c_str(), iPrecision, BASE10);
+    SetIsInteger(false);
 }
 
-LispInt CalculatePrecision(const LispChar * aString,LispInt aBasePrecision,LispInt aBase, LispBoolean& aIsFloat)
+LispInt CalculatePrecision(const LispChar * aString,LispInt aBasePrecision,LispInt aBase, bool& aIsFloat)
 {
   const LispChar * ptr = aString;
   while (*ptr)
@@ -1275,12 +1232,12 @@ FND_2:
         -- sig_digits;  // this is when we have "1.000001" where "." is not a digit, so need to decrement
     }
     // ok, so we need to represent MAX(aPrecision,sig_digits) digits in base aBase
-    aIsFloat = LispTrue;
-    return (LispInt) digits_to_bits(MAX(aBasePrecision,sig_digits), aBase);
+    aIsFloat = true;
+    return (LispInt) digits_to_bits(std::max(aBasePrecision,sig_digits), aBase);
   }
   else
   {
-    aIsFloat = LispFalse;
+    aIsFloat = false;
     return 0;
   }
 }
@@ -1290,7 +1247,7 @@ FND_2:
 void BigNumber::SetTo(const LispChar * aString,LispInt aBasePrecision,LispInt aBase)
 {//FIXME -- what?
 //  iPrecision = digits_to_bits(aBasePrecision,BASE10);
-  LispBoolean isFloat = 0;
+  bool isFloat = 0;
   LispInt digits = aBasePrecision;
   iPrecision = CalculatePrecision(aString,aBasePrecision,aBase, isFloat);
 
@@ -1305,7 +1262,7 @@ void BigNumber::SetTo(const LispChar * aString,LispInt aBasePrecision,LispInt aB
   if (!iNumber)   iNumber = NEW ANumber(digits);
   iNumber->SetPrecision(digits);
   iNumber->SetTo(aString,aBase);
- 
+
   SetIsInteger(!isFloat && iNumber->iExp == 0 && iNumber->iTensExp == 0);
 }
 
